/*= Rohde & Schwarz - Signal Generator SME ==================================*/
#include <formatio.h>
#include <userint.h>
#include <ansi_c.h>
#include "rssme.h"
#include "rssmeu.h"

/*= Global Variable Definitions =============================================*/
static int panelHandle[10];
static unsigned int instrHandle;

/*= Message Arrays ==========================================================*/
static char *initCtrlsHelpStr[3] = {"\n\
This control selects address of the device that is to be\n\
initialized.\n\n\
Valid Range:  Address 0 to 30\n\n\
Default Value :  Address 28","\n\
This control specifies if an ID Query is sent to the\n\
instrument during the initialization procedure.\n\n\
Valid Range:\nVI_OFF (0) - Skip Query\n\
VI_ON  (1) - Do Query (Default Value)","\n\
This control specifies if the instrument is to be reset to its\n\
power-on settings during the initialization procedure.\n\n\
Valid Range:\n\
VI_OFF (0) - Don't Reset\n\
VI_ON  (1) - Reset Device (Default Value)"};    
        
static char *configCtrlsHelpStr[6] = {"\n\
This control sets the RF output frequency.\n\n\
Valid Range:\n\
SME02 - 5.0 kHz to 1.5 GHz\n\
SME03 - 5.0 kHz to 3.0 GHz\n\
SME06 - 5.0 kHz to 6.0 GHz\n\n\
Default Value: 100.0 MHz","\n\
This control sets the RF output level.\n\n\
Valid Range: -144.0 dBm to +16.0 dBm\n\n\
Default Value:  -30.0 dBm","\n\
This control sets the amplitude modulation depth.\n\n\
Valid Range:  0.0 to 100.0 %\n\n\
Default Value: 30.0 %","\n\
This control sets the frequency of the LF Generator.\n\
Attempt to set other frequency then 400.0 Hz, 1.0 kHz, 3.0 kHz,\n\
or 15.0 kHz will cause an error \"Data out of range\" in case that\n\
neither SM-B2 nor SM-B6 option is present.\n\n\
Valid Range:\n\
Without option SM-B2 or SM-B6: 400.0 Hz, 1.0 kHz, 3.0 kHz, 15.0 kHz\n\
With option SM-B2: 0.1 Hz to 500.0 kHz\n\
With option SM-B6: 0.1 Hz to 1.0 MHz\n\n\
Default Value:  1.0 kHz","\n\
This control selects which internal generator is source of the modulating\n\
signal. LF Generator 1 can be the standard generator or second\n\
optional generator (2nd option SM-B2). LF Generator 2 can be\n\
first optional generator (1st option SM-B2) or multifunction\n\
generator (option SM-B6).\n\n\
Valid Range:\n\
VI_OFF (0) - LF Generator 1 (Default Value)\n\
VI_ON  (1) - LF Generator 2 (Option SM-B2 or SM-B6)","\n\
This control selects the waveform shape of the LF Generator. In case of the\n\
standard generator (Without option SM-B2) the waveform is sine.\n\
Attempt to select other then sine waveform shape will cause an \n\
error \"Data out of range\" in case that neither SM-B2 nor SM-B6 \n\
option is present.\n\n\
Valid Range:\n\
0 - Sine (Default Value)\n\
1 - Square (option SM-B2 or SM-B6)\n\
2 - Triangle (option SM-B2 or SM-B6)\n\
3 - Periodic Noise (option SM-B2 or SM-B6)\n\
4 - Sawtooth (option SM-B2 or SM-B6)"};

static char backgroundPnlHelpStr[] = {"\n\
This is a simple demo program using the Rohde&Schwarz SME\n\
Signal Generator instrument driver provided in the package."};

static char initPnlHelpStr[] = {"\n\
This panel performs the following initialization actions:\n\n\
- Opens a session to the Default Resource Manager resource and a\n\
session to the specified device using the interface and address\n\
specified in the Resource_Name control.\n\n\
- Performs an identification query on the Instrument.\n\n\
- Resets the instrument to a known state.\n\n\
- Sends initialization commands to the instrument that set any\n\
necessary programmatic variables.\n\n\
- Returns an Instrument Handle which is used to differentiate\n\
between different sessions of this instrument driver.\n\n\
- Each time this function is invoked a Unique Session is opened.\n\
It is possible to have more than one session open for the same\n\
resource."};

static char configPnlHelpStr[] = {"\n\
This panel configures the instrument to output an amplitude modulated signal.\n\n\
A call to Output Amplitude Modulated Signal function of the\n\
Rohde&Schwarz SME Signal Generator instrument driver is made when the\n\
\"Configure\" button is pressed."};

/*= Main Function ===========================================================*/

void main ()
{
    panelHandle[BCKGRND] = LoadPanel (0, "rssmeu.uir", BCKGRND);
    panelHandle[CONFIG] = LoadPanel (panelHandle[BCKGRND], "rssmeu.uir", CONFIG);
    panelHandle[INIT] = LoadPanel (panelHandle[BCKGRND], "rssmeu.uir", INIT);
    DisplayPanel (panelHandle[BCKGRND]);
    DisplayPanel (panelHandle[INIT]);
    RunUserInterface ();
}

/*===========================================================================*/
/* Function: Quit Application                                                */
/* Purpose:  This is a callback function of all Quit buttons of this         */
/*           application. It closes the instrument and quits the user        */
/*           interface.                                                      */
/*===========================================================================*/
int quitApplication (int panel, int control, int event, void *callbackData, int eventData1, 
    int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            if (ConfirmPopup ("Warning", "Do you really want to quit?")) {
                if (panel != panelHandle[INIT]) {
                    rssme_close (instrHandle);
                }   
                QuitUserInterface (0);
            }               
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button closes the instrument session and exits this example.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Initialize Instrument                                           */
/* Purpose:  This is a callback function of the Continue button on the       */
/*           Initialize panel. It initializes the instrument and switches to */
/*           the panel Configure.                                            */
/*===========================================================================*/
int initInstrument (int panel, int control, int event, void *callbackData, int eventData1, 
    int eventData2)
{
    short addr, rst, id;
    char instrDescr[20];

    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle[INIT], INIT_ADDRESS, &addr);
            GetCtrlVal (panelHandle[INIT], INIT_ID, &id);
            GetCtrlVal (panelHandle[INIT], INIT_RST, &rst);
            Fmt (instrDescr, "GPIB::%d[b2]", addr);
            if (rssme_init (instrDescr, id, rst, &instrHandle)) {
                MessagePopup ("Error", "Initialization error. Check your connections and make sure you have the right GPIB address.");
                return (0);            
            }
            else {
                HidePanel (panelHandle[INIT]);
                DisplayPanel (panelHandle[CONFIG]);
            }
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button causes the instrument to be initialized.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Control Help                                                    */
/* Purpose:  This is a callback function of all controls.                    */
/*           On the right mouse-click on the control a help window           */
/*           describing its purpose is displayed.                            */
/*===========================================================================*/
int controlHelp (int panel, int control, int event, void *callbackData, int eventData1, 
    int eventData2)
{
    if (event == EVENT_RIGHT_CLICK) {
        if (panel == panelHandle[INIT]) {
            MessagePopup ("Help", initCtrlsHelpStr[control-4]);
        }
        if (panel == panelHandle[CONFIG]) {     
            MessagePopup ("Help", configCtrlsHelpStr[control-4]);
        }
    }
    return 0;
}

/*===========================================================================*/
/* Function: Configure Instrument                                            */
/* Purpose:  This is a callback function of the Configure button on the      */
/*           Configure panel. The instrument is configured using the         */
/*           Output Amplitude Modulated Signal function of the Rohde&Schwarz */
/*           SME Signal Generator instrument driver.                         */
/*===========================================================================*/
int configInstrument (int panel, int control, int event, void *callbackData, int eventData1, 
    int eventData2)
{
    double carrFreq, carrLevel, carrDepth, LFFreq;
    int LFSource, LFShape;
    int uir_err = 0;
    char errMessage[256];

    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle[CONFIG], CONFIG_CARR_FREQ, &carrFreq);
            GetCtrlVal (panelHandle[CONFIG], CONFIG_CARR_LEVEL, &carrLevel);
            GetCtrlVal (panelHandle[CONFIG], CONFIG_DEPTH, &carrDepth);
            GetCtrlVal (panelHandle[CONFIG], CONFIG_LF_FREQ, &LFFreq);
            GetCtrlVal (panelHandle[CONFIG], CONFIG_LF_SOURCE, &LFSource);
            GetCtrlVal (panelHandle[CONFIG], CONFIG_LF_SHAPE, &LFShape);
            
            if ((uir_err = rssme_applicOutpAmplModul (instrHandle, carrFreq, carrLevel, 
                carrDepth, LFSource, LFFreq, LFShape)) != 0) {
                if (uir_err >= 0xBFFF0000)
                    quitApplication (panel, control, EVENT_COMMIT, callbackData, eventData1, 
                        eventData2);    
                else {
                    if (uir_err == 0xBFFC0002) {
                        MessagePopup ("Error", "Carrier Frequency out of range");
                        return (0);
                    }
                    else if (uir_err == 0xBFFC0003) {
                        MessagePopup ("Error", "Carrier Level out of range");
                        return (0);
                    }
                    else if (uir_err == 0xBFFC0004) {
                        MessagePopup ("Error", "Depth out of range");
                        return (0);
                    }
                    else if (uir_err == 0xBFFC0005) {
                        MessagePopup ("Error", "Internal Source out of range");
                        return (0);
                    }
                    else if (uir_err == 0xBFFC0006) {
                        MessagePopup ("Error", "LF Generator Frequency out of range");
                        return (0);
                    }
                    else if (uir_err == 0xBFFC0007) {
                        MessagePopup ("Error", "LF Generator Shape out of range");
                        return (0);
                    }
                    else {
                        rssme_error_message (instrHandle, uir_err, errMessage);
                        MessagePopup ("Error", errMessage);
                        return (0);
                    }
                }   
            }               
            break;              
            
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button will configure the instrument to output an AM signal.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Panel Help                                                      */
/* Purpose:  This is a callback function of the menu bar. It displays a help */
/*           window describing panel being used.                             */
/*===========================================================================*/
void panelHelp (int menuBar, int menuItem, void *callbackData, int panel)
{
    if (panel == panelHandle[BCKGRND])
        MessagePopup ("Help", backgroundPnlHelpStr);
    if (panel == panelHandle[INIT])
        MessagePopup ("Help", initPnlHelpStr);
    if (panel == panelHandle[CONFIG])
        MessagePopup ("Help", configPnlHelpStr);
        
    return;
}

/*= End =====================================================================*/
