/*****************************************************************************/
/*   Copyright 1996 National Instruments Corporation.  All Rights Reserved.  */
/*****************************************************************************/
#include <utility.h>
#include <formatio.h>
#include <visa.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "rssme.h"

#define RSSME_REVISION     "Rev 2.2, 11/1999, CVI 5.0.1" /*  Instrument driver revision */
#define BUFFER_SIZE         2048L                        /* Buffer size */

/*===========================================================================*/
/*= Rohde & Schwarz SME02, SME03, & SME06 Signal Generators =================*/
/* LabWindows/CVI 3.1 Instrument Driver                                      */
/* Original Release: October 1996                                            */
/* By: Michal Harhaj                                                         */
/*     Ph. +42 69 699 4882  Fax  +42 69 6919597  Email Michal.Harhaj@vsb.cz  */
/*                                                                           */
/* For LabWindows Technical Support in the United States call:               */
/*     National Instruments, Austin, Texas                                   */
/*     Ph (800) 433-3488    Fax (512) 794-5678                               */
/*                                                                           */
/* Modification History:                                                     */
/* December 1999:        Rev 2.2, 12/1999, CVI 5.0.1                         */
/*                       - Changed data type ViChar [] to ViString for input */
/*                         parameters. This is nessecary for VEE.            */
/* Modified by: Zdenek Rykala, ELCOM, a.s., CZ                               */
/*																			 */
/* November 1999:        Rev 2.1, 11/1999, CVI 5.0.1                         */
/*                       - Update to the latest VXIpnp standard              */
/*                         Changed following function names:                 */
/*                         rssme_errorQuery to rssme_error_query             */
/*                         rssme_errorMessage to rssme_error_message         */
/*                         rssme_selfTest to rssme_self_test                 */
/*                         rssme_revisionQuery to rssme_revision_query       */
/*                       - Removed warnings                                  */
/* Modified by: Zdenek Rykala, ELCOM, a.s., CZ                               */
/*                                                                           */
/* January 1999:         Rev 2.0, 03/1999, CVI 5.0.1                         */
/*                       Added New functions (fast configuration functions   */
/*                       with a single parameter). Functions added at the    */
/*                       request of Rohde & Schwarz.                         */
/*                                                                           */
/*         Quick Functions                                                   */
/*            RF Frequency                          rssme_qRFfrequency       */
/*            RF Level                              rssme_qRFlevel           */
/*            RF Level Offset                       rssme_qRFlevOffset       */
/*            RF Start Stop Frequency               rssme_qRFstartStopFreq   */
/*            RF Center Span Frequency              rssme_qRFcenterSpanFreq  */
/*            RF Sweep Step                         rssme_qRFsweepStep       */
/*            LF Frequency                          rssme_qLFfreq            */
/*            LF Voltage                            rssme_qLFvolt            */
/*            AM Modulation Depth                   rssme_qAMmodDep          */
/*            FM Deviation                          rssme_qFMdev             */
/*            Phase Mod Deviation                   rssme_qPHMdev            */
/*            Pulse Modulator Delay                 rssme_qPulsModDelay      */
/*            Pulse Modulator Period                rssme_qPulsModPeriod     */
/*            Pulse Modulator Width                 rssme_qPulsModWidth      */
/*            Stereo Modulation Deviation           rssme_qStereoModDev      */
/*            Stereo Modulation Audio Freq          rssme_qStereoModAudioFreq*/
/*                                                                           */
/* Modified by: Michal Harhaj, ELCOM, a.s., CZ                               */
/*																			 */
/* June 1997:            Modified the #defines for the constants for         */
/*                       status codes for parameters 9 to 15                 */
/*                       Modified by: TKS, National Instruments              */
/*                                                                           */
/*===========================================================================*/

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/
static ViString attrArr[] = {"AUTO","FIX"};
static ViString unitArr[] = {"DBM","DBW","DBMW","DBUW","DBV","DBMV","DBUV","V"};
static ViString stateArr[] = {"OFF","ON"};
static ViString onceArr[] = {"ONCE","ON","OFF"};
static ViString AMSourceArr [] = {"INT1","INT2","EXT"};
static ViString FMPMSourceArr [] = {"INT","EXT1","EXT2"};
static ViString couplingArr [] = {"AC","DC"};
static ViString extImpedanceArr[] = {"100KOHM","600OHM"};
static ViString pulMExtImpArr [] = {"10E3","50"};
static ViString polarityArr[] = {"NORM","INV"};
static ViString pulsePerFreqArr[] = {":SOUR:PULS:PER",":SOUR:PULM:INT:FREQ"};
static ViString sweBlankTmArr[] = {"NORM","LONG"};
static ViString lstSeqArr[] = {":SOUR:LIST",":SYST:MSEQ"};
static ViString spacingArr[] = {"LIN","LOG"};
static ViString freqSweModeArr[] = {":SOUR:FREQ:MODE CW",":SOUR:FREQ:MODE SWE;:SOUR:SWE:MODE AUTO",
            ":SOUR:FREQ:MODE SWE;:SOUR:SWE:MODE MAN",":SOUR:FREQ:MODE SWE;:SOUR:SWE:MODE STEP"};
static ViString powSweModeArr[] = {":SOUR:POW:MODE FIX",":SOUR:POW:MODE SWE;:SOUR:SWE:POW:MODE AUTO",
            ":SOUR:POW:MODE SWE;:SOUR:SWE:POW:MODE MAN",":SOUR:POW:MODE SWE;:SOUR:SWE:POW:MODE STEP"};
static ViString phaseUnitArr[] = {"RAD","DEG"};
static ViString stpArr[] = {"*SAV","*RCL"};
static ViString stepModeArr[] = {"AUTO","STEP"};
static ViString trigSourceArr[] = {"AUTO","EXT","SING"};
static ViString trigSlopeArr[] = {"POS","NEG","EITH"};
static ViString trigAbortArr[] = {"SWE","LIST","MSEQ","DM"};
static ViString sourceArr[] = {"INT","EXT"};
static ViString LFFreqModeArr[] = {"FIX","SWE"};
static ViString LFFreqInputModeArr[] = {"FIX","MAN"};
static ViString LFFunctionArr[] = {"SIN","SQU","TRI","PRN","SAWT"};
static ViString LFSweModeArr[] = {":SOUR2:FREQ:MODE CW",":SOUR2:FREQ:MODE SWE;:SOUR2:SWE:MODE AUTO",
            ":SOUR2:FREQ:MODE SWE;:SOUR2:SWE:MODE MAN",":SOUR2:FREQ:MODE SWE;:SOUR2:SWE:MODE STEP"};
static ViString stepConfigArr[] = {":SOUR:FREQ:STEP",":SOUR:FREQ:STEP",":SOUR:POW:STEP",":SOUR:POW:STEP"};
static ViString stepExeArr[] = {":SOUR:FREQ:MODE FIX;:SOUR:FREQ",":SOUR:FREQ:MODE SWE;:SOUR:FREQ",
            ":SOUR:POW:MODE FIX;:SOUR:POW",":SOUR:POW:MODE SWE;:SOUR:POW"};
static ViString directArr[] = {"UP","DOWN"};
static ViString ALCBwidArr[] = {"100KHZ","500KHZ"};
static ViString PMBwidArr[] = {"100KHZ","2MHZ"};
static ViString preemphArr[] = {"OFF","50US","75US"};
static ViString stereoARITypeArr[] = {"OFF","DK","BK"};
static ViString stereoARIBKArr[] = {"A","B","C","D","E","F"};
static ViString stereoModeArr[] = {"ARI","AUD;AUD:MODE RIGH","AUD;AUD:MODE LEFT","AUD;AUD:MODE REL",
            "AUD;AUD:MODE REML"};
static ViString stereoOutArr[] = {"MPX","PIL"};
static ViString VORDirArr[] = {"FROM","TO"};
static ViString VORILSSourArr[] = {"INT2","INT2,EXT"};
static ViString VORModeArr[] = {"STAT OFF","MODE NORM;STAT ON","MODE VAR;STAT ON","MODE SUBC;STAT ON",
            "MODE FMS;STAT ON"};
static ViString GSDirArr[] = {"UP","DOWN"};
static ViString ILSParamArr[] = {"DEPT","CURR","LOG"};
static ViString GSModeArr[] = {"NORM","ULOB","LLOB"};
static ViString locDirArr[] = {"LEFT","RIGH"};
static ViString locModeArr[] = {"NORM","LLOB","RLOB"};
static ViString MBEFreqArr[] = {"400HZ","1300HZ","3000HZ"};
static ViString recallArr[] = {"EXCL","INCL"};
static ViString PRBSArr[] = {"9","15","20","21","23"};
static ViString clkModeArr[] = {"BIT","SYMB"};
static ViString clkSourceArr[] = {"COUP","INT"};
static ViString attenModeArr[] = {":SOUR:DM:DATA:ALEV:MODE NORM;:SOUR:DM:GMSK:GSL OFF",
            ":SOUR:DM:DATA:ALEV:MODE MAX;:SOUR:DM:GMSK:GSL OFF",":SOUR:DM:GMSK:GSL ON"};
static ViString dataTypeArr[] = {"DATA","ATT","BURS"};
static ViString xmemModeArr[] = {"ALL","DATA"};
static ViString DMSourceArr[] = {"STAT OFF","SOUR EXT;STAT ON","SOUR PRBS;STAT ON","SOUR DATA;STAT ON"};
static ViString GMSKStandardArr[] = {"CDPD","DSRR","DSRR4K","GSM","MC9","MD24N","MD24W","MD36N","MD36W",
            "MD48N","MD48W","MD80N","MD80W","MD96N","MD96W","MD100W","MD120W","MD160","MD192","MOB"};
static ViString GMSKBitrateArr[] = {"2.4","2.5","3.0","3.6","4.0","4.8","5.0","6.0","7.2","8.0","9.6",
            "10.0","12.0","14.4","16.0","19.2","20.0","24.0","28.8","32.0","38.4","40.0","64.0","76.8",
            "80.0","160.0","270.833","512.0","1000.0"};
static ViString GMSKFilter[] = {"0.2","0.25","0.3","0.4","0.5"};
static ViString GFSKStandardArr[] = {"DECT","CT2","CT3"};
static ViString GFSKFilter[] = {"0.4","0.5","0.6","0.7"};
static ViString GFSKDeviationArr[] = {"14.4","18","20","25.2","160","202","259","288","317","403"};
static ViString QPSKStandardArr[] = {"ADC","PDC","TFTS","TETR","APCO25","MSAT","INM"};
static ViString QPSKFilter[] = {"OFF,0","COS,0.2","COS,0.35","SCOS,0.35","COS,0.4","SCOS,0.4","COS,0.5",
            "SCOS,0.5","COS,0.6","SCOS,0.6"};
static ViString QPSKTypeArr[] = {"QPSK","OQPS","PI4Q","PI4D"};
static ViString FSKStandardArr[] = {"POCS512","POCS1200","POCS2400","CITY512","CITY1200","CITY2400",
            "FLEX1600","FLEX3200"};
static ViString FSKFilter[] = {"OFF,0","GAUS,2.73","BESS,1.22","BESS,2.44"};
static ViString FSK4StandardArr[] = {"ERM","APCO","MOD","FLEX3200","FLEX6400"};
static ViString FSK4Filter[] = {"SCOS,0.2","COS,0.2","BESS,1.22","BESS,1.25","BESS,2.44"};
static ViString FSK4CodingArr[] = {"ERM","APCO","MOD","FLEX"};
static ViString FFSKStandardArr[] = {"POCSAG"};
static ViString FFSKDeviationArr[] = {"1.5","2.0","3.0","3.5","4.0","4.5"};
static ViString ERMESAlphaNameArr[] = {"FOX","ALPHA","LONG","USER1","USER2","USER3"};
static ViString ERMESPOCSAGTypeArr[] = {"TONE","NUM","ALPH"};
static ViString FLEXPhasArr[] = {"A","B","C","D","AB","AC","AD","BC","BD","CD","ABC","ABD","ACD","BCD","ABCD"};
static ViString FLEXPOCSAGAlphaNameArr[] = {"FOX","ALPHA","USER1","USER2","USER3","USER4"};
static ViString FLEXBinNameArr[] = {"USER1","USER2"};
static ViString FLEXTypeArr[] = {"TONE","NUM","SNUM","ALPH","SEC","BIN"};
static ViString FLEXModulationArr[] = {"1600,FSK2","3200,FSK2","3200,FSK4","6400,FSK4"};
static ViString FLEXBinTypeArr[] = {"LEFT","RIGH","TRAN","THE"};
static ViString POCSAGModulArr[] = {"FSK","FFSK"};
static ViString POCSAGDevArr[] = {"1.5","2.0","3.0","3.5","4.0","4.5"};
static ViString POCSAGBitrateArr[]= {"512","1200","2400"};
static ViString POCSAGToneArr[] = {"A","B","C","D"};
static ViString POCSAGSyncWord[] = {"POCS","INF"};
static ViString DMRecalculArr[] = {"ERM","FLEX","POCS"};
static ViString DMStateArr[] = {":SOUR:ERM:STAT OFF;:SOUR:FLEX:STAT OFF;:SOUR:POCS:STAT OFF",
            ":SOUR:FLEX:STAT OFF;:SOUR:POCS:STAT OFF;:SOUR:ERM:STAT ON",
            ":SOUR:POCS:STAT OFF;:SOUR:ERM:STAT OFF;:SOUR:FLEX:STAT ON",
            ":SOUR:ERM:STAT OFF;:SOUR:FLEX:STAT OFF;:SOUR:POCS:STAT ON"};
static ViString optionsList[] = {"SM-B1","SM-B2","SM-B2","SM-B3","SM-B4","SM-B5",
            "SM-B6","SM-B8","SM-B9","SME-B11","SME-B12","SME-B41","SME-B42"};
static ViString instrVersionList[] = {"SME02","SME03","SME06"};
static ViReal64 maxFrequencyList[] = {1.5E9, 3E9, 6E9};
static ViString zeroOrTwo[] = {"0", "2"};
static ViString oneOrTwo[] = {"1", "2"};

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* rssme_stringValPair is used in the rssme_error_message function           */
/* rssme_statusDataRanges is used to track session dependent status & ranges */
/* buffer is used to write to and to read from the instrument                */
/* option is used to store instrument options information                    */
/* maxFrequency stores maximum frequency (dependent upon instrument version) */
/*===========================================================================*/
static ViByte buffer[BUFFER_SIZE];

typedef struct  rssme_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  rssme_tStringValPair;

struct rssme_statusDataRanges {
    ViInt16 option;
    ViReal64 maxFrequency;
    ViChar instrDriverRevision[256];
};

typedef struct rssme_statusDataRanges *rssme_instrRange;

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViBoolean rssme_invalidViBooleanRange (ViBoolean val);
ViBoolean rssme_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max);
ViBoolean rssme_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean rssme_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViStatus rssme_initCleanUp (ViSession openRMSession, ViPSession openInstrSession, ViStatus currentStatus);
ViStatus rssme_writeFromFile (ViSession instrSession, ViString filename, ViUInt32 writeBytes, ViPUInt32 retCount);

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT UTILITY ROUTINE DECLARATIONS =======================*/
/*****************************************************************************/
ViStatus rssme_defaultInstrSetup (ViSession openInstrSession);
ViStatus rssme_sysStatus (ViSession instrSession);
ViStatus rssme_queryInstrVersion (ViSession instrSession, ViPReal64 maxFreq);
ViStatus rssme_queryInstrOption (ViSession instrSession, ViInt16 queryBit, ViPInt16 bitValue);
ViStatus rssme_confOPT (ViSession instrSession, ViString optsList);

/*****************************************************************************/
/*= USER-CALLABLE FUNCTIONS (Exportable Functions) ==========================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_init (ViRsrc resourceName, ViBoolean IDQuery,
                    ViBoolean reset, ViPSession instrSession)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViSession rmSession = 0;
    ViUInt32 retCnt = 0;

    /*- Check input parameter ranges ----------------------------------------*/
    if (rssme_invalidViBooleanRange (IDQuery))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViBooleanRange (reset))
        return VI_ERROR_PARAMETER3;

    /*- Open instrument session ---------------------------------------------*/
    if ((rssme_status = viOpenDefaultRM (&rmSession)) < 0)
        return rssme_status;

    if ((rssme_status = viOpen (rmSession, resourceName, VI_NULL, VI_NULL, instrSession)) < 0) {
        viClose (rmSession);
        return rssme_status;
    }

    /*- Configure VISA Formatted I/O ----------------------------------------*/
    if ((rssme_status = viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, 10000)) < 0)
            return rssme_initCleanUp (rmSession, instrSession, rssme_status);
    if ((rssme_status = viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, 4000)) < 0)
            return rssme_initCleanUp (rmSession, instrSession, rssme_status);
    if ((rssme_status = viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,
                            VI_FLUSH_ON_ACCESS)) < 0)
            return rssme_initCleanUp (rmSession, instrSession, rssme_status);
    if ((rssme_status = viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,
                            VI_FLUSH_ON_ACCESS)) < 0)
            return rssme_initCleanUp (rmSession, instrSession, rssme_status);

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery) {
        if ((rssme_status = viWrite (*instrSession, (ViBuf)"*IDN?", 5, &retCnt)) < 0)
            return rssme_initCleanUp (rmSession, instrSession, rssme_status);
        if ((rssme_status = viRead (*instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
            return rssme_status;

        Scan (buffer, "Rohde&Schwarz,SME");
        if (NumFmtdBytes () != 17)
            return rssme_initCleanUp (rmSession, instrSession, VI_ERROR_FAIL_ID_QUERY);
    }

    /*- Reset instrument ----------------------------------------------------*/
    if (reset) {
        if ((rssme_status = rssme_reset (*instrSession)) < 0)
            return rssme_initCleanUp (rmSession, instrSession, rssme_status);
    }
    else  /*- Send Default Instrument Setup ---------------------------------*/
        if ((rssme_status = rssme_defaultInstrSetup (*instrSession)) < 0)
            return rssme_initCleanUp (rmSession, instrSession, rssme_status);

    return rssme_status;
}

/*****************************************************************************/
/*= USER-CALLABLE INSTRUMENT-DEPENDENT ROUTINES =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Output Amplitude Modulated Signal                               */
/* Purpose:  This function represents a high-level approach to the instrument*/
/* driver. It demonstrates how to create an application function which       */
/* contains a set of low-level functions. E.g. this function fully           */
/* configures the instrument to output an amplitude modulated signal.        */
/* This function performs the following operations:                          */
/* 1. Resets the instrument                                                  */
/* 2. Sets the RF output frequency                                           */
/* 3. Sets the RF output level                                               */
/* 4. Configures the Automatic Level Control                                 */
/* 5. Configures the internal LF generator (source of the modulating signal) */
/* 6. Configures the amplitude modulation of the RF output signal            */
/* 7. Turns on the instrument's RF output.                                   */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_applicOutpAmplModul (ViSession instrSession, ViReal64 carrierFreq,
    ViReal64 carrierLevel, ViReal64 depth, ViBoolean LFSource, ViReal64 LFFreq,
    ViInt16 LFShape)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViReal64 junk = 0.0;

    if (rssme_invalidViReal64Range (carrierFreq, 5E3, 6E9))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViReal64Range (carrierLevel, -144.0, 16.0))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViReal64Range (depth, 0.0, 100.0))
        return VI_ERROR_PARAMETER4;
    if (rssme_invalidViBooleanRange (LFSource))
        return VI_ERROR_PARAMETER5;

    if (LFSource == VI_OFF) {
        if ((rssme_status = rssme_queryInstrOption (instrSession, 0x4, &rssme_optPresent)) < 0)
            return rssme_status;
        if (rssme_optPresent == VI_FALSE) {
            switch ((ViInt16)LFFreq) {
                case 400:
                case 1000:
                case 3000:
                case 15000:
                    if (modf (LFFreq, &junk) != 0)
                        return VI_ERROR_PARAMETER6;
                    break;
                default:
                    return VI_ERROR_PARAMETER6;
            }
            if (LFShape != 0)
                return VI_ERROR_PARAMETER7;
        }
        else {
            if (rssme_invalidViReal64Range (LFFreq, 0.1, 5E5))
                return VI_ERROR_PARAMETER6;
            if (rssme_invalidViInt16Range (LFShape, 0, 4))
                return VI_ERROR_PARAMETER7;
        }
    }
    else {
        if ((rssme_status = rssme_queryInstrOption (instrSession, 0x2, &rssme_optPresent)) < 0)
            return rssme_status;
        if (rssme_optPresent == VI_ON) {
            if (rssme_invalidViReal64Range (LFFreq, 0.1, 5E5))
                return VI_ERROR_PARAMETER6;
            if (rssme_invalidViInt16Range (LFShape, 0, 4))
                return VI_ERROR_PARAMETER7;
        }
        else {
            if ((rssme_status = rssme_queryInstrOption (instrSession, 0x20, &rssme_optPresent)) < 0)
                return rssme_status;
            if (rssme_optPresent == VI_ON) {
                if (rssme_invalidViReal64Range (LFFreq, 0.1, 1E6))
                    return VI_ERROR_PARAMETER6;
                if (rssme_invalidViInt16Range (LFShape, 0, 4))
                    return VI_ERROR_PARAMETER7;
            }
            else {
                rssme_status = rssme_WARN_NUSP_OPTION;
                return rssme_status;
            }
        }
    }

    /* 1. Reset the instrument  -------------------------------------------------*/
    if ((rssme_status = rssme_reset (instrSession)) < 0)
        return rssme_status;

    /* 2. Set the RF output frequency -------------------------------------------*/
    if ((rssme_status = rssme_confCarrFreq (instrSession, carrierFreq, 0.0, VI_OFF)) < 0)
        return rssme_status;

    /* 3. Set the RF output level -----------------------------------------------*/
    if ((rssme_status = rssme_confCarrLevel (instrSession, carrierLevel, 0.0, 16.0, 0, VI_OFF, VI_OFF)) < 0)
        return rssme_status;

    /* 4. Configure Automatic Level Control -------------------------------------*/
    if ((rssme_status = rssme_confALC (instrSession, VI_ON, 1, VI_OFF)) < 0)
        return rssme_status;

    /* 5. Configure Internal Modulating Source (LF Generator) -------------------*/
    if (LFSource == VI_OFF) {
        if ((rssme_status = rssme_confLFGen (instrSession, VI_OFF, 1.0, LFFreq, LFShape, 1E3, 0, VI_OFF)) < 0)
            return rssme_status;
    }
    else
        if ((rssme_status = rssme_confLFGen (instrSession, VI_ON, 1.0, 1E3, 0, LFFreq, LFShape, VI_OFF)) < 0)
            return rssme_status;

    /* 6. Configure Amplitude Modulation ----------------------------------------*/
    if ((rssme_status = rssme_confAmplModul (instrSession, VI_ON, (ViInt16)LFSource, depth, VI_OFF, VI_OFF)) < 0)
        return rssme_status;

    /* 7. Turn on the instrument's RF output -----------------------------------*/
    if ((rssme_status = rssme_outState (instrSession, VI_ON, VI_OFF)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Frequency                                                       */
/* Purpose:  This function sets frequency of the instrument's RF output.     */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confCarrFreq (ViSession instrSession, ViReal64 freq,
    ViReal64 offset, ViBoolean mode)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViReal64 rssme_maxFreq = 6E9;

    if ((rssme_status = rssme_queryInstrVersion (instrSession, &rssme_maxFreq)) < 0)
        return rssme_status;
    if (rssme_invalidViReal64Range (freq, 5E3, rssme_maxFreq))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViReal64Range (offset, -50.0E9, 50.0E9))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViBooleanRange (mode))
        return VI_ERROR_PARAMETER4;

    if (mode == VI_ON) {
        if ((rssme_status = viPrintf (instrSession, ":SOUR:FREQ:MODE SWE;:SOUR:FREQ:MAN %f", freq)) < 0)
            return rssme_status;
    }
    else
        if ((rssme_status = viPrintf (instrSession, ":SOUR:FREQ:MODE CW;:SOUR:FREQ:CW %f;OFFS %f",
            freq, offset)) < 0)
            return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Level                                                           */
/* Purpose:  This function controls the RF output level.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confCarrLevel (ViSession instrSession, ViReal64 level, ViReal64 offset,
    ViReal64 limit, ViInt16 unit, ViBoolean amplMode, ViBoolean attenuator)
{
    ViStatus rssme_status = VI_SUCCESS;

    if (rssme_invalidViReal64Range (offset, -100.0, 100.0))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViInt16Range (unit, 0, 7))
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViBooleanRange (amplMode))
        return VI_ERROR_PARAMETER6;
    if (rssme_invalidViBooleanRange (attenuator))
        return VI_ERROR_PARAMETER7;
    if (unit == 0) {
        if (rssme_invalidViReal64Range (level, -144.0, 16.0))
            return VI_ERROR_PARAMETER2;
        if (rssme_invalidViReal64Range (limit, -144.0, 16.0))
            return VI_ERROR_PARAMETER4;
    }

    if ((rssme_status = viPrintf (instrSession, ":OUTP:AMOD %s", attrArr[attenuator])) < 0)
        return rssme_status;

    if (amplMode == VI_ON) {
        if ((rssme_status = viPrintf (instrSession, ":SOUR:POW:MODE SWE;:SOUR:POW %f%s",
                level, unitArr[unit])) < 0)
            return rssme_status;
    }
    else
        if ((rssme_status = viPrintf (instrSession, ":SOUR:POW:MODE FIX;:SOUR:POW %f%s;:SOUR:POW:OFFS %f;LIM %f%s",
                level, unitArr[unit], offset, limit, unitArr[unit])) < 0)
            return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: ALC (Automatic Level Control)                                   */
/* Purpose:  This function enables and configures the automatic level control*/
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confALC (ViSession instrSession, ViBoolean state, ViInt16 autobw,
    ViBoolean bandwidth)
{
    ViStatus rssme_status = VI_SUCCESS;

    if (rssme_invalidViBooleanRange (state))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViInt16Range (autobw, 0, 2))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViBooleanRange (bandwidth))
        return VI_ERROR_PARAMETER4;

    if (autobw == 2) {
        if ((rssme_status = viPrintf (instrSession, ":POW:ALC:BWID:AUTO OFF;:POW:ALC:BWID %s;:POW:ALC:STAT %s",
                ALCBwidArr[bandwidth], stateArr[state])) < 0)
            return rssme_status;
    }
    else
        if ((rssme_status = viPrintf (instrSession, ":POW:ALC:BWID:AUTO %s;:POW:ALC:STAT %s",
                onceArr[autobw], stateArr[state])) < 0)
            return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: User Correction                                                 */
/* Purpose:  This function is used to create and activate lists in which     */
/*           arbitrary RF frequencies are assigned level correction values.  */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confUserCorr (ViSession instrSession, ViBoolean state,
    ViInt16 action, ViString name, ViReal64 _VI_FAR freq[], ViReal64 _VI_FAR power[], ViInt32 length)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt32 i = 0;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    ViReal64 rssme_maxFreq = 6E9;

    if (rssme_invalidViBooleanRange (state))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViInt16Range (action, 0, 2))
        return VI_ERROR_PARAMETER3;
    if (StringLength (name) > NAME_LEN) {
        rssme_status = VI_ERROR_PARAMETER4;
        return rssme_status;
    }
    if (rssme_invalidViInt32Range (length, 1, 160))
        return VI_ERROR_PARAMETER7;
    if ((rssme_status = rssme_queryInstrVersion (instrSession, &rssme_maxFreq)) < 0)
        return rssme_status;

    switch (action) {
        case 0:
            if ((rssme_status = viPrintf (instrSession, ":SOUR:CORR:CSET:SEL \"%s\"", name)) < 0)
                return rssme_status;

            Fmt (buffer, "%s<:SOUR:CORR:CSET:DATA:FREQ ");
            bytesFmt = NumFmtdBytes ();

            for (i = 0; i < length; i++) {
                if (rssme_invalidViReal64Range (freq[i], 5E3, rssme_maxFreq))
                    return VI_ERROR_PARAMETER5;
                Fmt (buffer, "%s[a]<%f,", freq[i]);
                bytesFmt += NumFmtdBytes ();
            }

            if ((rssme_status = viWrite (instrSession, buffer, bytesFmt - 1, &retCnt)) < 0)
                return rssme_status;

            Fmt (buffer, "%s<:SOUR:CORR:CSET:DATA:POW ");
            bytesFmt = NumFmtdBytes ();

            for (i = 0; i < length; i++) {
                if (rssme_invalidViReal64Range (power[i], -40.0, 6.0))
                    return VI_ERROR_PARAMETER6;
                Fmt (buffer, "%s[a]<%f,", power[i]);
                bytesFmt += NumFmtdBytes ();
            }

            if ((rssme_status = viWrite (instrSession, buffer, bytesFmt - 1, &retCnt)) < 0)
                return rssme_status;
            break;

        case 1:
            if ((rssme_status = viPrintf (instrSession, ":SOUR:CORR:CSET:DEL \"%s\"", name)) < 0)
                return rssme_status;
            break;
        case 2:
            if ((rssme_status = viPrintf (instrSession, ":SOUR:CORR:CSET:SEL \"%s\"", name)) < 0)
                return rssme_status;
            break;
        }

    if ((rssme_status = viPrintf (instrSession, ":SOUR:CORR:STAT %s", stateArr[state])) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Amplitude Modulation                                            */
/* Purpose:  This function configures amplitude modulation.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confAmplModul (ViSession instrSession, ViBoolean state,
    ViInt16 source, ViReal64 depth, ViBoolean extCoupling, ViBoolean polarity)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;

    if (rssme_invalidViBooleanRange (state))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViInt16Range (source, 0, 2))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViReal64Range (depth, 0.0, 100.0))
        return VI_ERROR_PARAMETER4;
    if (rssme_invalidViBooleanRange (extCoupling))
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViBooleanRange (polarity))
        return VI_ERROR_PARAMETER6;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x2, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF)
        if ((rssme_status = rssme_queryInstrOption (instrSession, 0x20, &rssme_optPresent)) < 0)
            return rssme_status;

    if ((rssme_optPresent == VI_OFF) && (source == 1)) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    if ((rssme_status = viPrintf (instrSession, ":SOUR:AM:DEPT %f;SOUR %s;:SOUR:AM:STAT %s;EXT:COUP %s;:SOUR:AM:POL %s",
            depth, AMSourceArr[source], stateArr[state], couplingArr[extCoupling], polarityArr[polarity])) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Frequency Modulation                                            */
/* Purpose:  This function configures frequency modulation.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confFreqModul (ViSession instrSession, ViBoolean state1,
    ViBoolean state2, ViReal64 deviation1, ViReal64 deviation2, ViInt16 source1,
    ViInt16 source2, ViBoolean extCoupling1, ViBoolean extCoupling2, ViInt16 preemph)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViReal64 rssme_maxFreq = 6E9;
    
    if ((rssme_status = rssme_queryInstrVersion (instrSession, &rssme_maxFreq)) < 0)
        return rssme_status;

    if (rssme_invalidViBooleanRange (state1))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViBooleanRange (state2))
        return VI_ERROR_PARAMETER3;
    if (rssme_maxFreq == 1.5E9) {  /* SME02 */
        if (rssme_invalidViReal64Range (deviation1, 0.0, 1E6))
            return VI_ERROR_PARAMETER4;
        if (rssme_invalidViReal64Range (deviation2, 0.0, 1E6))
            return VI_ERROR_PARAMETER5;
    }           
    if (rssme_maxFreq == 3E9) {    /* SME03 */
        if (rssme_invalidViReal64Range (deviation1, 0.0, 2E6))
            return VI_ERROR_PARAMETER4;
        if (rssme_invalidViReal64Range (deviation2, 0.0, 2E6))
            return VI_ERROR_PARAMETER5;
    }           
    if (rssme_maxFreq == 6E9) {    /* SME06 */
        if (rssme_invalidViReal64Range (deviation1, 0.0, 4E6))
            return VI_ERROR_PARAMETER4;
        if (rssme_invalidViReal64Range (deviation2, 0.0, 4E6))
            return VI_ERROR_PARAMETER5;
    }           
    if (rssme_invalidViInt16Range (source1, 0, 2))
        return VI_ERROR_PARAMETER6;
    if (rssme_invalidViInt16Range (source2, 0, 2))
        return VI_ERROR_PARAMETER7;
    if (rssme_invalidViBooleanRange (extCoupling1))
        return VI_ERROR_PARAMETER8;
    if (rssme_invalidViBooleanRange (extCoupling2))
        return VI_ERROR_INSTR_PARAMETER9;
    if (rssme_invalidViInt16Range (preemph, 0, 2))
        return VI_ERROR_INSTR_PARAMETER10;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x2, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF)
        if ((rssme_status = rssme_queryInstrOption (instrSession, 0x20, &rssme_optPresent)) < 0)
            return rssme_status;

    if ((rssme_optPresent == VI_OFF) && (state2 == VI_ON) && (source2 == 0)) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    if ((rssme_status = viPrintf (instrSession, ":SOUR:FM1:PRE %s;SOUR %s;DEV %f;STAT %s;:SOUR:FM2:SOUR %s;DEV %f;STAT %s;EXT1:COUP %s;:SOUR:FM:EXT2:COUP %s",
            preemphArr[preemph], FMPMSourceArr[source1], deviation1, stateArr[state1],
            FMPMSourceArr[source2], deviation2, stateArr[state2], couplingArr[extCoupling1],
            couplingArr[extCoupling2])) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Phase Modulation                                                */
/* Purpose:  This function configures phase modulation.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confPhasModul (ViSession instrSession, ViBoolean state1,
    ViReal64 deviation1, ViBoolean state2, ViReal64 deviation2, ViBoolean unit,
    ViInt16 source1, ViInt16 source2, ViBoolean extCoupling1, ViBoolean extCoupling2)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;

    if (rssme_invalidViBooleanRange (unit))
        return VI_ERROR_PARAMETER6;
    if (rssme_invalidViBooleanRange (state1))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViBooleanRange (state2))
        return VI_ERROR_PARAMETER3;
    if (unit == VI_OFF) {
        if (rssme_invalidViReal64Range (deviation1, -2*PI, 2*PI))
            return VI_ERROR_PARAMETER4;
        if (rssme_invalidViReal64Range (deviation2, -2*PI, 2*PI))
            return VI_ERROR_PARAMETER5;
    }
    else {
        if (rssme_invalidViReal64Range (deviation1, -360.0, 360.0))
            return VI_ERROR_PARAMETER4;
        if (rssme_invalidViReal64Range (deviation2, -360.0, 360.0))
            return VI_ERROR_PARAMETER5;
    }
    if (rssme_invalidViInt16Range (source1, 0, 2))
        return VI_ERROR_PARAMETER7;
    if (rssme_invalidViInt16Range (source2, 0, 2))
        return VI_ERROR_PARAMETER8;
    if (rssme_invalidViBooleanRange (extCoupling1))
        return VI_ERROR_INSTR_PARAMETER9;
    if (rssme_invalidViBooleanRange (extCoupling2))
        return VI_ERROR_INSTR_PARAMETER10;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x2, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF)
        if ((rssme_status = rssme_queryInstrOption (instrSession, 0x20, &rssme_optPresent)) < 0)
            return rssme_status;

    if ((rssme_optPresent == VI_OFF) && (state2 == VI_ON) && (source2 == 0)) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    if ((rssme_status = viPrintf (instrSession, ":SOUR:PM1:SOUR %s;DEV %f%s;STAT %s;:SOUR:PM2:SOUR %s;DEV %f%s;STAT %s;EXT1:COUP %s;:SOUR:PM:EXT2:COUP %s",
            FMPMSourceArr[source1], deviation1, phaseUnitArr[unit], stateArr[state1],
            FMPMSourceArr[source2], deviation2, phaseUnitArr[unit], stateArr[state2],
            couplingArr[extCoupling1], couplingArr[extCoupling2])) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Pulse Modulation                                                */
/* Purpose:  This function configures pulse modulation.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confPulsModul (ViSession instrSession, ViBoolean state,
    ViBoolean polarity, ViBoolean extImp, ViReal64 pulseWidth, ViReal64 pulseDelay,
    ViReal64 pulsePerFreq, ViBoolean pulseParam, ViBoolean pulseShape, ViBoolean source)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;

    if (rssme_invalidViBooleanRange (state))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViBooleanRange (polarity))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViBooleanRange (extImp))
        return VI_ERROR_PARAMETER4;
    if (rssme_invalidViReal64Range (pulseWidth, 20E-9, 1.0))
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViReal64Range (pulseDelay, 20E-9, 1.0))
        return VI_ERROR_PARAMETER6;
    if (rssme_invalidViBooleanRange (pulseParam))
        return VI_ERROR_PARAMETER8;
    if (pulseParam == VI_OFF) {
        if (rssme_invalidViReal64Range (pulsePerFreq, 100E-9, 85.0))
            return VI_ERROR_PARAMETER7;
    }
    else
        if (rssme_invalidViReal64Range (pulsePerFreq, 0.01176, 10E6))
            return VI_ERROR_PARAMETER7;
    if (rssme_invalidViBooleanRange (pulseShape))
        return VI_ERROR_INSTR_PARAMETER9;
    if (rssme_invalidViBooleanRange (source))
        return VI_ERROR_INSTR_PARAMETER10;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x8, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF)
        if ((rssme_status = rssme_queryInstrOption (instrSession, 0x40, &rssme_optPresent)) < 0)
            return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x16, &rssme_optPresent)) < 0)
        return rssme_status;

    if (rssme_optPresent == VI_OFF) {
        if (source == VI_OFF) {
            rssme_status = rssme_WARN_NUSP_OPTION;
            return rssme_status;
        }
        if ((rssme_status = viPrintf (instrSession, ":SOUR:PULM:POL %s;SOUR EXT;STAT %s",
                    polarityArr[polarity], stateArr[state])) < 0)
                return rssme_status;
    }
    else {
        if ((rssme_status = viPrintf (instrSession, ":SOUR:PULM:EXT:IMP %s;:SOUR:PULM:POL %s;SOUR %s;:SOUR:PULS:DEL %f;WIDT %f;:SOUR:PULS:DOUB %s;%s %f;:SOUR:PULM:STAT %s",
                    pulMExtImpArr[extImp], polarityArr[polarity], sourceArr[source],
                    pulseDelay, pulseWidth, stateArr[pulseShape], pulsePerFreqArr[pulseParam],
                    pulsePerFreq, stateArr[state])) < 0)
                return rssme_status;
    }

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Stereo Modulation                                               */
/* Purpose:  This function configures stereo modulation.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confStereoModul (ViSession instrSession, ViBoolean state,
    ViReal64 dev, ViReal64 audioFreq, ViInt16 preemph, ViBoolean pilotState,
    ViReal64 pilotDev, ViReal64 pilotPhase, ViBoolean phaseUnit, ViInt16 mode, ViReal64 ARIDev,
    ViInt16 ARIBK, ViInt16 ARIIdent, ViBoolean stereoOut)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;

    if (rssme_invalidViBooleanRange (state))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViReal64Range (dev, 0.0, 1E5))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViReal64Range (audioFreq, 0.1, 15E3))
        return VI_ERROR_PARAMETER4;
    if (rssme_invalidViInt16Range (preemph, 0, 2))
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViBooleanRange (pilotState))
        return VI_ERROR_PARAMETER6;
    if (rssme_invalidViReal64Range (pilotDev, 0.0, 1E4))
        return VI_ERROR_PARAMETER7;
    if (rssme_invalidViBooleanRange (phaseUnit))
        return VI_ERROR_INSTR_PARAMETER9;
    if (phaseUnit == VI_OFF) {
        if (rssme_invalidViReal64Range (pilotPhase, 0.0, 2*PI))
            return VI_ERROR_PARAMETER8;
    }
    else
        if (rssme_invalidViReal64Range (pilotPhase, 0.0, 360.0))
            return VI_ERROR_PARAMETER8;
    if (rssme_invalidViInt16Range (mode, 0, 4))
        return VI_ERROR_INSTR_PARAMETER10;
    if (rssme_invalidViReal64Range (ARIDev, 0.0, 1E4))
        return VI_ERROR_INSTR_PARAMETER11;
    if (rssme_invalidViInt16Range (ARIBK, 0, 5))
        return VI_ERROR_INSTR_PARAMETER12;
    if (rssme_invalidViInt16Range (ARIIdent, 0, 2))
        return VI_ERROR_INSTR_PARAMETER13;
    if (rssme_invalidViBooleanRange (stereoOut))
        return VI_ERROR_INSTR_PARAMETER14;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x20, &rssme_optPresent)) < 0)
        return rssme_status;

    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    if ((rssme_status = viPrintf (instrSession, ":SOUR:STER:DEV %f;AUD:FREQ %f;PRE %s;:SOUR:STER:PIL:DEV %f;PHAS %f%s;STAT %s;:SOUR:STER:ARI:DEV %f;TYPE %s;BK %s;:SOUR:STER:SIGN %s;:SOUR:STER:STAT %s;:OUTP2:SOUR:STER %s",
            dev, audioFreq, preemphArr[preemph], pilotDev, pilotPhase, phaseUnitArr[phaseUnit],
            stateArr[pilotState], ARIDev, stereoARITypeArr[ARIIdent], stereoARIBKArr[ARIBK],
            stereoModeArr[mode], stateArr[state], stereoOutArr[stereoOut])) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: VOR Modulation                                                  */
/* Purpose:  This function configures the VOR modulation.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confVORModul (ViSession instrSession, ViInt16 mode,
    ViReal64 bearAngle, ViBoolean unit, ViBoolean direction, ViReal64 VARFreq,
    ViReal64 VARDepth, ViReal64 subFreq, ViReal64 subDepth, ViBoolean comState,
    ViReal64 comFreq, ViReal64 comDepth, ViReal64 refDev, ViBoolean source)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;

    if (rssme_invalidViInt16Range (mode, 0, 4))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViBooleanRange (unit))
        return VI_ERROR_PARAMETER4;
    if (unit == VI_OFF) {
        if (rssme_invalidViReal64Range (bearAngle, 0.0, 2*PI))
            return VI_ERROR_PARAMETER3;
    }
    else
        if (rssme_invalidViReal64Range (bearAngle, 0.0, 360.0))
            return VI_ERROR_PARAMETER3;
    if (rssme_invalidViBooleanRange (direction))
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViReal64Range (VARFreq, 20.0, 40.0))
        return VI_ERROR_PARAMETER6;
    if (rssme_invalidViReal64Range (VARDepth, 0.0, 100.0))
        return VI_ERROR_PARAMETER7;
    if (rssme_invalidViReal64Range (subFreq, 5E3, 15E3))
        return VI_ERROR_PARAMETER8;
    if (rssme_invalidViReal64Range (subDepth, 0.0, 100.0))
        return VI_ERROR_INSTR_PARAMETER9;
    if (rssme_invalidViBooleanRange (comState))
        return VI_ERROR_INSTR_PARAMETER10;
    if (rssme_invalidViReal64Range (comFreq, 0.1, 2E4))
        return VI_ERROR_INSTR_PARAMETER11;
    if (rssme_invalidViReal64Range (comDepth, 0.0, 100.0))
        return VI_ERROR_INSTR_PARAMETER12;
    if (rssme_invalidViReal64Range (refDev, 0.0, 960.0))
        return VI_ERROR_INSTR_PARAMETER13;
    if (rssme_invalidViBooleanRange (source))
        return VI_ERROR_INSTR_PARAMETER14;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x20, &rssme_optPresent)) < 0)
        return rssme_status;

    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    if ((rssme_status = viPrintf (instrSession, ":SOUR:VOR:BANG %f%s;DIR %s;:SOUR:VOR:VAR:DEPT %f;FREQ %f;:SOUR:VOR:SUBC:DEPT %f;FREQ %f;:SOUR:VOR:REF %f;COM:DEPT %f;FREQ %f;STAT %s;:SOUR:VOR:SOUR %s;%s",
            bearAngle, phaseUnitArr[unit], VORDirArr[direction], VARDepth, VARFreq, subDepth,
            subFreq, refDev, comDepth, comFreq, stateArr[comState], VORILSSourArr[source],
            VORModeArr[mode])) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: ILS-GS Modulation                                               */
/* Purpose:  This function configures the ILS-GS modulation.                 */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confILSGSModul (ViSession instrSession, ViBoolean state,
    ViBoolean direction, ViReal64 upFreq, ViReal64 downFreq, ViReal64 phase,
    ViBoolean unit, ViInt16 signalType, ViInt16 DDMParam, ViReal64 DDMValue,
    ViBoolean comState, ViReal64 comFreq, ViReal64 comDepth, ViBoolean source,
    ViReal64 sumOfDepth)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;

    if (rssme_invalidViBooleanRange (state))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViBooleanRange (direction))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViReal64Range (upFreq, 60.0, 120.0))
        return VI_ERROR_PARAMETER4;
    if (rssme_invalidViReal64Range (downFreq, 100.0, 200.0))
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViBooleanRange (unit))
        return VI_ERROR_PARAMETER7;
    if (unit == VI_OFF) {
        if (rssme_invalidViReal64Range (phase, 0.0, 120*PI/180))
            return VI_ERROR_PARAMETER6;
    }
    else
        if (rssme_invalidViReal64Range (phase, 0.0, 120.0))
            return VI_ERROR_PARAMETER6;
    if (rssme_invalidViInt16Range (signalType, 0, 2))
        return VI_ERROR_PARAMETER8;
    switch (DDMParam) {
        case 0:
            if (rssme_invalidViReal64Range (DDMValue, -0.8, 0.8))
                return VI_ERROR_INSTR_PARAMETER10;
        break;
        case 1:
            if (rssme_invalidViReal64Range (DDMValue, -698.0, 698.0))
                return VI_ERROR_INSTR_PARAMETER10;
        break;
        case 2:
            if (rssme_invalidViReal64Range (DDMValue, -999.9, 999.9))
                return VI_ERROR_INSTR_PARAMETER10;
        break;
        default:
            return VI_ERROR_INSTR_PARAMETER9;
    }
    if (rssme_invalidViBooleanRange (comState))
        return VI_ERROR_INSTR_PARAMETER11;
    if (rssme_invalidViReal64Range (comFreq, 0.1, 2E4))
        return VI_ERROR_INSTR_PARAMETER12;
    if (rssme_invalidViReal64Range (comDepth, 0.0, 100.0))
        return VI_ERROR_INSTR_PARAMETER13;
    if (rssme_invalidViBooleanRange (source))
        return VI_ERROR_INSTR_PARAMETER14;
    if (rssme_invalidViReal64Range (sumOfDepth, 0.0, 100.0))
        return VI_ERROR_INSTR_PARAMETER15;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x20, &rssme_optPresent)) < 0)
        return rssme_status;

    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    if ((rssme_status = viPrintf (instrSession, ":SOUR:ILS:GS:DDM:%s %f;DIR %s;:SOUR:ILS:GS:SOD %f;ULOB %f;LLOB %f;PHAS %f%s;MODE %s;:SOUR:ILS:COM:FREQ %f;DEPT %f;STAT %s;:SOUR:ILS:SOUR %s;TYPE GS;STAT %s",
            ILSParamArr[DDMParam], DDMValue, GSDirArr[direction], sumOfDepth, upFreq, downFreq,
            phase, phaseUnitArr[unit], GSModeArr[signalType], comFreq, comDepth, stateArr[comState],
            VORILSSourArr[source], stateArr[state])) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: ILS-Localizer Modulation                                        */
/* Purpose:  This function configures the ILS-Localizer modulation.          */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confILSLocModul (ViSession instrSession, ViBoolean state,
    ViBoolean direction, ViReal64 leftFreq, ViReal64 rightFreq, ViReal64 phase,
    ViBoolean unit, ViInt16 signalType, ViInt16 DDMParam, ViReal64 DDMValue,
    ViBoolean comState, ViReal64 comFreq, ViReal64 comDepth, ViBoolean source,
    ViReal64 sumOfDepth)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;

    if (rssme_invalidViBooleanRange (state))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViBooleanRange (direction))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViReal64Range (leftFreq, 60.0, 120.0))
        return VI_ERROR_PARAMETER4;
    if (rssme_invalidViReal64Range (rightFreq, 100.0, 200.0))
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViBooleanRange (unit))
        return VI_ERROR_PARAMETER7;
    if (unit == VI_OFF) {
        if (rssme_invalidViReal64Range (phase, 0.0, 120*PI/180))
            return VI_ERROR_PARAMETER6;
    }
    else
        if (rssme_invalidViReal64Range (phase, 0.0, 120.0))
            return VI_ERROR_PARAMETER6;
    if (rssme_invalidViInt16Range (signalType, 0, 2))
        return VI_ERROR_PARAMETER8;
    switch (DDMParam) {
        case 0:
            if (rssme_invalidViReal64Range (DDMValue, -0.4, 0.4))
                return VI_ERROR_INSTR_PARAMETER10;
        break;
        case 1:
            if (rssme_invalidViReal64Range (DDMValue, -387.0, 387.0))
                return VI_ERROR_INSTR_PARAMETER10;
        break;
        case 2:
            if (rssme_invalidViReal64Range (DDMValue, -999.9, 999.9))
                return VI_ERROR_INSTR_PARAMETER10;
        break;
        default:
            return VI_ERROR_INSTR_PARAMETER9;
    }
    if (rssme_invalidViBooleanRange (comState))
        return VI_ERROR_INSTR_PARAMETER11;
    if (rssme_invalidViReal64Range (comFreq, 0.1, 2E4))
        return VI_ERROR_INSTR_PARAMETER12;
    if (rssme_invalidViReal64Range (comDepth, 0.0, 100.0))
        return VI_ERROR_INSTR_PARAMETER13;
    if (rssme_invalidViBooleanRange (source))
        return VI_ERROR_INSTR_PARAMETER14;
    if (rssme_invalidViReal64Range (sumOfDepth, 0.0, 100.0))
        return VI_ERROR_INSTR_PARAMETER15;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x20, &rssme_optPresent)) < 0)
        return rssme_status;

    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    if ((rssme_status = viPrintf (instrSession, ":SOUR:ILS:LOC:DDM:%s %f;DIR %s;:SOUR:ILS:LOC:SOD %f;LLOB %f;RLOB %f;PHAS %f%s;MODE %s;:SOUR:ILS:COM:FREQ %f;DEPT %f;STAT %s;:SOUR:ILS:SOUR %s;TYPE LOC;STAT %s",
            ILSParamArr[DDMParam], DDMValue, locDirArr[direction], sumOfDepth, leftFreq, rightFreq,
            phase, phaseUnitArr[unit], locModeArr[signalType], comFreq, comDepth, stateArr[comState],
            VORILSSourArr[source], stateArr[state])) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Marker Beacon Modulation                                        */
/* Purpose:  This function configures the Marker Beacon Modulation.          */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confMBeModul (ViSession instrSession, ViBoolean state,
    ViInt16 freq, ViReal64 depth, ViBoolean comState, ViReal64 comFreq, ViReal64 comDepth)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;

    if (rssme_invalidViBooleanRange (state))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViInt16Range (freq, 0, 2))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViReal64Range (depth, 0.0, 100.0))
        return VI_ERROR_PARAMETER4;
    if (rssme_invalidViBooleanRange (comState))
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViReal64Range (comFreq, 0.1, 2E4))
        return VI_ERROR_PARAMETER6;
    if (rssme_invalidViReal64Range (comDepth, 0.0, 100.0))
        return VI_ERROR_PARAMETER7;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x20, &rssme_optPresent)) < 0)
        return rssme_status;

    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    if ((rssme_status = viPrintf (instrSession, ":SOUR:MBE:COM:FREQ %f;DEPT %f;STAT %s;:SOUR:MBE:MARK:FREQ %s;DEPT %f;:SOUR:MBE:STAT %s",
            comFreq, comDepth, stateArr[comState], MBEFreqArr[freq], depth, stateArr[state])) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: GMSK                                                            */
/* Purpose:  This function configures and enables the GMSK (Gaussian Minimum */
/*           Shift Keying) modulation.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confDMGMSK (ViSession instrSession, ViInt16 source, ViInt16 standard,
    ViInt16 bitrate, ViInt16 filter, ViBoolean polarity, ViBoolean encoder)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;

    if (rssme_invalidViInt16Range (source, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViInt16Range (standard, 0, 20))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViInt16Range (bitrate, 0, 29))
        return VI_ERROR_PARAMETER4;
    if (rssme_invalidViInt16Range (filter, 0, 4))
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViBooleanRange (polarity))
        return VI_ERROR_PARAMETER6;
    if (rssme_invalidViBooleanRange (encoder))
        return VI_ERROR_PARAMETER7;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x200, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    if (standard) {
        Fmt (buffer, "%s<:SOUR:DM:TYPE GMSK;:SOUR:DM:GMSK:STAN %s;:SOUR:DM:%s",
            GMSKStandardArr[standard - 1], DMSourceArr[source]);
        bytesFmt = NumFmtdBytes ();
    }
    else {
        Fmt (buffer, "%s<:SOUR:DM:TYPE GMSK;:SOUR:DM:GMSK:BRAT %sKB/S;DCOD %s;FILT %s;POL %s;:SOUR:DM:%s",
            GMSKBitrateArr[bitrate], stateArr[encoder], GMSKFilter[filter], polarityArr[polarity],
            DMSourceArr[source]);
        bytesFmt = NumFmtdBytes ();
    }

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: GFSK                                                            */
/* Purpose:  This function configures and enables the GMSK (Gaussian         */
/*           Frequency Shift Keying) modulation.                             */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confDMGFSK (ViSession instrSession, ViInt16 source, ViInt16 standard,
    ViReal64 bitrate, ViInt16 filter, ViBoolean polarity, ViInt16 deviation)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;

    if (rssme_invalidViInt16Range (source, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViInt16Range (standard, 0, 3))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViReal64Range (bitrate, 10.0, 585.0))
        if (rssme_invalidViReal64Range (bitrate,  640.0, 1170.0))
            return VI_ERROR_PARAMETER4;
    if (rssme_invalidViInt16Range (filter, 0, 3))
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViBooleanRange (polarity))
        return VI_ERROR_PARAMETER6;
    if (rssme_invalidViInt16Range (deviation, 0, 9))
        return VI_ERROR_PARAMETER7;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x200, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    if (standard) {
        Fmt (buffer, "%s<:SOUR:DM:TYPE GFSK;:SOUR:DM:GFSK:STAN %s;:SOUR:DM:%s",
            GFSKStandardArr[standard - 1], DMSourceArr[source]);
        bytesFmt = NumFmtdBytes ();
    }
    else {
        Fmt (buffer, "%s<:SOUR:DM:TYPE GFSK;:SOUR:DM:GFSK:BRAT %fKB/S;FILT %s;POL %s;DEV %sKHZ;:SOUR:DM:%s",
            bitrate, GFSKFilter[filter], polarityArr[polarity], GFSKDeviationArr[deviation],
            DMSourceArr[source]);
        bytesFmt = NumFmtdBytes ();
    }

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: QPSK                                                            */
/* Purpose:  This function configures and enables the QPSK (Quad Phase Shift */
/*           Keying) modulation.                                             */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confDMQPSK (ViSession instrSession, ViInt16 source, ViInt16 standard,
    ViReal64 bitrate, ViInt16 filter, ViBoolean polarity, ViInt16 type, ViInt16 coding)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;

    if (rssme_invalidViInt16Range (source, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViInt16Range (standard, 0, 7))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViReal64Range (bitrate, 1.0, 24.3))
        if (rssme_invalidViReal64Range (bitrate,  27.0, 48.6))
            return VI_ERROR_PARAMETER4;
    if (rssme_invalidViInt16Range (filter, 0, 9))
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViBooleanRange (polarity))
        return VI_ERROR_PARAMETER6;
    if (rssme_invalidViInt16Range (type, 0, 3))
        return VI_ERROR_PARAMETER7;
    if (rssme_invalidViInt16Range (coding, 0, 6))
        return VI_ERROR_PARAMETER8;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x200, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    if (standard) {
        Fmt (buffer, "%s<:SOUR:DM:TYPE QPSK;:SOUR:DM:QPSK:STAN %s;:SOUR:DM:%s",
            QPSKStandardArr[standard - 1], DMSourceArr[source]);
        bytesFmt = NumFmtdBytes ();
    }
    else {
        Fmt (buffer, "%s<:SOUR:DM:TYPE QPSK;:SOUR:DM:QPSK:BRAT %fKB/S;FILT %s;POL %s;TYPE %s;COD %s;:SOUR:DM:%s",
            bitrate, QPSKFilter[filter], polarityArr[polarity], QPSKTypeArr[type],
            QPSKStandardArr[coding], DMSourceArr[source]);
        bytesFmt = NumFmtdBytes ();
    }

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: FSK                                                             */
/* Purpose:  This function configures and enables the FSK (Frequency Shift   */
/*           Keying) modulation.                                             */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confDMFSK (ViSession instrSession, ViInt16 source, ViInt16 standard,
    ViReal64 bitrate, ViInt16 filter, ViBoolean polarity, ViReal64 deviation)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;

    if (rssme_invalidViInt16Range (source, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViInt16Range (standard, 0, 8))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViInt16Range (filter, 0, 3))
        return VI_ERROR_PARAMETER5;
    if (filter) {
        if (rssme_invalidViReal64Range (bitrate, 0.05, 90.0))
            return VI_ERROR_PARAMETER4;
    }
    else
        if (rssme_invalidViReal64Range (bitrate, 0.05, 1900.0))
            return VI_ERROR_PARAMETER4;
    if (rssme_invalidViBooleanRange (polarity))
        return VI_ERROR_PARAMETER6;
    if (rssme_invalidViReal64Range (deviation, 0.01, 400.0))
        return VI_ERROR_PARAMETER7;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x200, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    if (standard) {
        Fmt (buffer, "%s<:SOUR:DM:TYPE FSK;:SOUR:DM:FSK:STAN %s;:SOUR:DM:%s",
            FSKStandardArr[standard - 1], DMSourceArr[source]);
        bytesFmt = NumFmtdBytes ();
    }
    else {
        Fmt (buffer, "%s<:SOUR:DM:TYPE FSK;:SOUR:DM:FSK:BRAT %fKB/S;FILT %s;POL %s;DEV %fKHZ;:SOUR:DM:%s",
            bitrate, FSKFilter[filter], polarityArr[polarity], deviation, DMSourceArr[source]);
        bytesFmt = NumFmtdBytes ();
    }

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: FSK4                                                            */
/* Purpose:  This function configures and enables the FSK4 (Frequency Shift  */
/*           Keying) modulation.                                             */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confDMFSK4 (ViSession instrSession, ViInt16 source, ViInt16 standard,
    ViReal64 bitrate, ViInt16 filter, ViBoolean polarity, ViReal64 deviation, ViInt16 coding)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;

    if (rssme_invalidViInt16Range (source, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViInt16Range (standard, 0, 5))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViReal64Range (bitrate, 1.0, 24.3))
        if (rssme_invalidViReal64Range (bitrate,  27.0, 48.6))
            return VI_ERROR_PARAMETER4;
    if (rssme_invalidViInt16Range (filter, 0, 6))
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViBooleanRange (polarity))
        return VI_ERROR_PARAMETER6;
    if (rssme_invalidViReal64Range (deviation, 0.01, 400.0))
        return VI_ERROR_PARAMETER7;
    if (rssme_invalidViInt16Range (coding, 0, 3))
        return VI_ERROR_PARAMETER8;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x200, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    if (standard) {
        Fmt (buffer, "%s<:SOUR:DM:TYPE FSK4;:SOUR:DM:FSK4:STAN %s;:SOUR:DM:%s",
            FSK4StandardArr[standard - 1], DMSourceArr[source]);
        bytesFmt = NumFmtdBytes ();
    }
    else {
        Fmt (buffer, "%s<:SOUR:DM:TYPE FSK4;:SOUR:DM:FSK4:BRAT %fKB/S;FILT %s;POL %s;DEV %fKHZ;COD %s;:SOUR:DM:%s",
            bitrate, FSK4Filter[filter], polarityArr[polarity], deviation, FSK4CodingArr[coding],
            DMSourceArr[source]);
        bytesFmt = NumFmtdBytes ();
    }

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: FFSK                                                            */
/* Purpose:  This function configures and enables the FFSK (Fast Frequency   */
/*           Shift Keying modulation.                                        */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confDMFFSK (ViSession instrSession, ViInt16 source, ViInt16 standard,
    ViReal64 bitrate, ViInt16 deviation)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;

    if (rssme_invalidViInt16Range (source, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViInt16Range (standard, 0, 1))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViReal64Range (bitrate, 0.05, 90.0))
        return VI_ERROR_PARAMETER4;
    if (rssme_invalidViInt16Range (deviation, 0, 5))
        return VI_ERROR_PARAMETER5;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x200, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    if (standard) {
        Fmt (buffer, "%s<:SOUR:DM:TYPE FFSK;:SOUR:DM:FFSK:STAN %s;:SOUR:DM:%s",
            FFSKStandardArr[standard - 1], DMSourceArr[source]);
        bytesFmt = NumFmtdBytes ();
    }
    else {
        Fmt (buffer, "%s<:SOUR:DM:TYPE FFSK;:SOUR:DM:FFSK:BRAT %fKB/S;DEV %sKHZ;:SOUR:DM:%s",
            bitrate, FFSKDeviationArr[deviation], DMSourceArr[source]);
        bytesFmt = NumFmtdBytes ();
    }

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Digital Modulation Setup                                        */
/* Purpose:  This function turns on or off a specified digital modulation    */
/*          (radiotelecommunication service) and defines its operating mode. */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confDMSetup (ViSession instrSession, ViInt16 modulType,
    ViInt16 outMode)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;

    if (rssme_invalidViInt16Range (modulType, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViInt16Range (outMode, 0, 2))
        return VI_ERROR_PARAMETER3;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x200, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }
    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x400, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }
    if (modulType == 2) {   /* FLEX */
        if ((rssme_status = rssme_queryInstrOption (instrSession, 0x800, &rssme_optPresent)) < 0)
            return rssme_status;
        if (rssme_optPresent == VI_OFF) {
            rssme_status = rssme_WARN_NUSP_OPTION;
            return rssme_status;
        }
    }
    if (modulType == 3) {   /* POCSAG */
        if ((rssme_status = rssme_queryInstrOption (instrSession, 0x1000, &rssme_optPresent)) < 0)
            return rssme_status;
        if (rssme_optPresent == VI_OFF) {
            rssme_status = rssme_WARN_NUSP_OPTION;
            return rssme_status;
        }
    }

    Fmt (buffer, "%s<:TRIG:DM:SOUR %s;%s", trigSourceArr[outMode], DMStateArr[modulType]);
    bytesFmt = NumFmtdBytes ();

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: ERMES Channel/Bit/Batch                                         */
/* Purpose:  This function configures the channel, bit and message sequence  */
/*           of ERMES.                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confDMERMESChanBitBatch (ViSession instrSession, ViInt16 channel,
    ViString message, ViInt32 errBitMask, ViString errBatchPos, ViInt32 errWordPos)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;

    if (rssme_invalidViInt16Range (channel, 0, 15))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViInt32Range (errBitMask, 0, 1073741823))
        return VI_ERROR_PARAMETER3;
    StringUpperCase (errBatchPos);
    if ((errBatchPos[0] < 'A')||(errBatchPos[0] > 'P'))
        return VI_ERROR_PARAMETER4;
    if (errBatchPos[0] == 'P') {
        if (rssme_invalidViInt32Range (errWordPos, 0, 189))
            return VI_ERROR_PARAMETER5;
    }
    else
        if (rssme_invalidViInt32Range (errWordPos, 0, 153))
            return VI_ERROR_PARAMETER5;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x200, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }
    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x400, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    Fmt (buffer, "%s<:SOUR:ERM:CHAN %i[b2];ERR:MASK %i[b4];BATC %s;WORD %i[b4];:SOUR:ERM:SEQ:DBAT %s",
        channel, errBitMask, errBatchPos, errWordPos, message);
    bytesFmt = NumFmtdBytes ();

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: ERMES Network/System                                            */
/* Purpose:  This function configures the network and system subsystems of   */
/*           ERMES.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confDMERMESNetSyst (ViSession instrSession, ViBoolean extTrafIndicator,
    ViBoolean bordAreaIndicator, ViInt32 freqSubsIndicator, ViInt32 day, ViInt32 hour,
    ViInt32 minute, ViInt32 zoneCode, ViInt32 operCode, ViInt32 pagingArea)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;

    if (rssme_invalidViBooleanRange (extTrafIndicator))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViBooleanRange (bordAreaIndicator))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViInt32Range (freqSubsIndicator, 0, 30))
        return VI_ERROR_PARAMETER4;
    if (rssme_invalidViInt32Range (day, 1, 31))
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViInt32Range (hour, 0, 24))
        return VI_ERROR_PARAMETER6;
    if (rssme_invalidViInt32Range (minute, 0, 59))
        return VI_ERROR_PARAMETER7;
    if (rssme_invalidViInt32Range (zoneCode, 200, 799))
        return VI_ERROR_PARAMETER8;
    if (rssme_invalidViInt32Range (operCode, 0, 7))
        return VI_ERROR_INSTR_PARAMETER9;
    if (rssme_invalidViInt32Range (pagingArea, 0, 63))
        return VI_ERROR_INSTR_PARAMETER10;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x200, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }
    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x400, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    Fmt (buffer, "%s<:SOUR:ERM:SI:ETI %s;BAI %s;FSI %i[b4];DOM %i[b4];TIME %i[b4],%i[b4];:SOUR:ERM:NINF:ZCO %i[b4];OPER %i[b4];PA %i[b4]",
        stateArr[extTrafIndicator], stateArr[bordAreaIndicator], freqSubsIndicator, day, hour,
        minute, zoneCode, operCode, pagingArea);
    bytesFmt = NumFmtdBytes ();

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: ERMES Message                                                   */
/* Purpose:  This function configures the message subsystem of ERMES.        */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confDMERMESMessage (ViSession instrSession, ViInt32 address,
    ViInt16 messageType, ViInt16 toneNumb, ViString numMessage, ViInt16 alphaName,
    ViString alphaMessage)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;

    if (rssme_invalidViInt32Range (address, 0, 262143))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViInt16Range (messageType, 0, 3))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViInt16Range (toneNumb, 0, 15))
        return VI_ERROR_PARAMETER4;
    if (StringLength (numMessage) > 16)
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViInt16Range (alphaName, 0, 5))
        return VI_ERROR_PARAMETER6;
    if (StringLength (alphaMessage) > 369)
        return VI_ERROR_PARAMETER7;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x200, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }
    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x400, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    if (messageType == 3)
        Fmt (buffer, "%s<:SOUR:ERM:MESS:IA %i[b4];TONE %i[b2];NUM \"%s\";ALPH \"%s\";ALPH:DATA \"%s\";:SOUR:ERM:MESS:CAT ALPH",
            address, toneNumb, numMessage, ERMESAlphaNameArr[alphaName], alphaMessage);
    else
        Fmt (buffer, "%s<:SOUR:ERM:MESS:IA %i[b4];TONE %i[b2];NUM \"%s\";ALPH \"%s\";CAT %s",
            address, toneNumb, numMessage, ERMESAlphaNameArr[alphaName], ERMESPOCSAGTypeArr[messageType]);
    bytesFmt = NumFmtdBytes ();

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: FLEX System/Bit                                                 */
/* Purpose:  This function configures the system and bit subsystems of the   */
/*           FLEX Radiocommunication Service.                                */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confDMFLEXSystBit (ViSession instrSession, ViInt16 modul,
    ViReal64 deviation, ViInt32 systCollapse, ViInt32 chanID, ViInt32 timeZone,
    ViInt32 year, ViInt32 month, ViInt32 day, ViInt32 hour, ViInt32 minute,
    ViReal64 bitMask, ViInt32 wordPos)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;

    if (rssme_invalidViInt16Range (modul, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViReal64Range (deviation, 2.0, 10.0))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViInt32Range (systCollapse, 0, 7))
        return VI_ERROR_PARAMETER4;
    if (rssme_invalidViInt32Range (chanID, 0, 511))
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViInt32Range (timeZone, 0, 31))
        return VI_ERROR_PARAMETER6;
    if (rssme_invalidViInt32Range (year, 0, 9999))
        return VI_ERROR_PARAMETER7;
    if (rssme_invalidViInt32Range (month, 1, 12))
        return VI_ERROR_PARAMETER8;
    if (rssme_invalidViInt32Range (day, 1, 31))
        return VI_ERROR_INSTR_PARAMETER9;
    if (rssme_invalidViInt32Range (hour, 0, 24))
        return VI_ERROR_INSTR_PARAMETER10;
    if (rssme_invalidViInt32Range (minute, 0, 59))
        return VI_ERROR_INSTR_PARAMETER11;
    if (rssme_invalidViReal64Range (bitMask, 0.0, 4294967295))
        return VI_ERROR_INSTR_PARAMETER12;
    if (rssme_invalidViInt32Range (wordPos, 0, 87))
        return VI_ERROR_INSTR_PARAMETER13;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x200, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }
    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x400, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }
    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x800, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    Fmt (buffer, "%s<:SOUR:FLEX:MOD %s;DEV %fKHZ;SI:COLL %i[b4];LCH %i[b4];TZON %i[b4];DATE %i[b4],%i[b4],%i[b4];TIME %i[b4],%i[b4];:SOUR:FLEX:ERR:MASK %f;WORD %i[b4]",
        FLEXModulationArr[modul], deviation, systCollapse, chanID, timeZone, year, month,
        day, hour, minute, bitMask, wordPos);
    bytesFmt = NumFmtdBytes ();

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: FLEX Select Message                                             */
/* Purpose:  This function configures the message subsystem of the FLEX      */
/*           Radiocommunication Service.                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confDMFLEXSelMessage (ViSession instrSession, ViString capsode,
    ViInt16 phase, ViInt32 toneNumb, ViBoolean adjPhase, ViBoolean adjFrame,
    ViInt32 repeat, ViBoolean messNumbering, ViBoolean mailFlag, ViInt16 messageType,
    ViString numMessage, ViInt16 alphaName, ViInt16 binName)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;

    if (rssme_invalidViInt16Range (phase, 0, 14))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViInt32Range (toneNumb, 0, 7))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViBooleanRange (adjPhase))
        return VI_ERROR_PARAMETER4;
    if (rssme_invalidViBooleanRange (adjFrame))
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViInt32Range (repeat, 0, 3))
        return VI_ERROR_PARAMETER6;
    if (rssme_invalidViBooleanRange (messNumbering))
        return VI_ERROR_PARAMETER7;
    if (rssme_invalidViBooleanRange (mailFlag))
        return VI_ERROR_PARAMETER8;
    if (rssme_invalidViInt16Range (messageType, 0, 5))
        return VI_ERROR_INSTR_PARAMETER9;
    if (StringLength (numMessage) > 41)
        return VI_ERROR_INSTR_PARAMETER10;
    if (rssme_invalidViInt16Range (alphaName, 0, 5))
        return VI_ERROR_INSTR_PARAMETER11;
    if (rssme_invalidViInt16Range (binName, 0, 1))
        return VI_ERROR_INSTR_PARAMETER12;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x200, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }
    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x400, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }
    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x800, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    Fmt (buffer, "%s<:SOUR:FLEX:MESS:CAPC \"%s\";TONE %i[b4];REP %i[b4];CAT %s;MNUM %s;MDR %s;:SOUR:FLEX:PHAS %s;PHAS:AUTO %s;:SOUR:FLEX:FCON:AUTO %s;:SOUR:FLEX:MESS:NUM \"%s\";ALPH \"%s\";BIN:SEL \"%s\"",
            capsode, toneNumb, repeat, FLEXTypeArr[messageType], stateArr[messNumbering],
            stateArr[mailFlag], FLEXPhasArr[phase], stateArr[adjPhase], stateArr[adjFrame],
            numMessage, FLEXPOCSAGAlphaNameArr[alphaName], FLEXBinNameArr[binName]);
    bytesFmt = NumFmtdBytes ();

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: FLEX Edit Message                                               */
/* Purpose:  This function allows the user to edit messages for the FLEX     */
/*           Radiocommunication Service.                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confDMFLEXEditMessage (ViSession instrSession, ViInt16 type,
    ViInt16 alphaName, ViString alphaMessage, ViInt32 frameStart, ViString frame,
    ViInt16 binName, ViString binMessage, ViInt16 binType, ViInt32 binLength)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;

    if (rssme_invalidViInt16Range (type, 0, 2))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViInt16Range (alphaName, 0, 5))
        return VI_ERROR_PARAMETER3;
    if (StringLength (alphaMessage) > 128)
        return VI_ERROR_PARAMETER4;
    if (rssme_invalidViInt32Range (frameStart, 0, 14))
        return VI_ERROR_PARAMETER5;
    if (StringLength (frame) > 128)
        return VI_ERROR_PARAMETER6;
    if (rssme_invalidViInt16Range (binName, 0, 1))
        return VI_ERROR_PARAMETER7;
    if (StringLength (binMessage) > 460)
        return VI_ERROR_PARAMETER8;
    if (rssme_invalidViInt16Range (binType, 0, 3))
        return VI_ERROR_INSTR_PARAMETER9;
    if (rssme_invalidViInt32Range (binLength, 1, 16))
        return VI_ERROR_INSTR_PARAMETER10;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x200, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }
    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x400, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }
    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x800, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    switch (type) {
        case (0):
            Fmt (buffer, "%s<:SOUR:FLEX:MESS:ALPH \"%s\";ALPH:DATA \"%s\"", FLEXPOCSAGAlphaNameArr[alphaName], alphaMessage);
        break;
        case (1):
            Fmt (buffer, "%s<:SOUR:FLEX:FCON \"%s\";:SOUR:FLEX:CYCL %i[b4]", frame, frameStart);
        break;
        case (2):
            Fmt (buffer, "%s<:SOUR:FLEX:MESS:BIN:SEL \"%s\";DATA \"%s\";BLEN %i[b4];TYPE %s",
                FLEXBinNameArr[binName], binMessage, binLength, FLEXBinTypeArr[binType]);
        break;
    }
    bytesFmt = NumFmtdBytes ();

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: POCSAG System                                                   */
/* Purpose:  This function configures the system of the POCSAG               */
/*           Radiocommunication Service.                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confDMPOCSAGSyst (ViSession instrSession, ViBoolean modulation,
    ViInt16 deviation, ViInt16 bitrate)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;

    if (rssme_invalidViBooleanRange (modulation))
        return VI_ERROR_PARAMETER2;
    if (modulation) {
        if (rssme_invalidViInt16Range (deviation, 0, 5))
            return VI_ERROR_PARAMETER3;
    }
    else
        if (rssme_invalidViInt16Range (deviation, 4, 5))
            return VI_ERROR_PARAMETER3;
    if (rssme_invalidViInt16Range (bitrate, 0, 2))
        return VI_ERROR_PARAMETER4;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x200, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }
    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x400, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }
    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x1000, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    Fmt (buffer, "%s<:SOUR:POCS:MOD %s;DEV %sKHZ;BRAT %s", POCSAGModulArr[modulation],
        POCSAGDevArr[deviation], POCSAGBitrateArr[bitrate]);
    bytesFmt = NumFmtdBytes ();

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: POCSAG Message                                                  */
/* Purpose:  This function configures the message subsystem of the POCSAG    */
/*           Radiocommunication Service.                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confDMPOCSAGMessage (ViSession instrSession, ViInt32 address,
    ViInt16 messageType, ViInt16 toneNumb, ViString numMessage, ViInt16 alphaName,
    ViString alphaMessage, ViBoolean syncWord, ViInt32 timeSlice)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;

    if (rssme_invalidViInt32Range (address, 0, 2097151))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViInt16Range (messageType, 0, 3))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViInt16Range (toneNumb, 0, 3))
        return VI_ERROR_PARAMETER4;
    if (StringLength (numMessage) > 41)
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViInt16Range (alphaName, 0, 5))
        return VI_ERROR_PARAMETER6;
    if (StringLength (alphaMessage) > 128)
        return VI_ERROR_PARAMETER7;
    if (rssme_invalidViBooleanRange (syncWord))
        return VI_ERROR_PARAMETER8;
    if (rssme_invalidViInt32Range (timeSlice, 2, 120))
        return VI_ERROR_INSTR_PARAMETER9;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x200, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }
    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x400, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }
    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x1000, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    if (messageType == 3)
        Fmt (buffer, "%s<:SOUR:POSC:MESS:ADDR %i[b4];TONE %s;SWOR %s;TSL %i[b4];ALPH \"%s\";DATA \"%s\";CAT ALPH;NUM \"%s\"",
            address, POCSAGToneArr[toneNumb], POCSAGSyncWord[syncWord], timeSlice,
            FLEXPOCSAGAlphaNameArr[alphaName], alphaMessage, numMessage);
    else
        Fmt (buffer, "%s<:SOUR:POCS:MESS:ALPH \"%s\";ADDR %i[b4];TONE %s;SWOR %s;TSL %i[b4];CAT %s;NUM \"%s\"",
            FLEXPOCSAGAlphaNameArr[alphaName], address, POCSAGToneArr[toneNumb], POCSAGSyncWord[syncWord],
            timeSlice, ERMESPOCSAGTypeArr[messageType], numMessage);
    bytesFmt = NumFmtdBytes ();

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Edit DM Data Catalog                                            */
/* Purpose:  This function allows the user to edit catalog of DM data lists  */
/*           stored in the instrument's memory.                              */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confDMEditCatalog (ViSession instrSession, ViInt16 action,
    ViChar _VI_FAR oldCatalog[], ViPInt32 oldOccupMem, ViPInt32 oldVacantMem, ViString listName,
    ViChar _VI_FAR newCatalog[], ViPInt32 newOccupMem, ViPInt32 newVacantMem)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViUInt32 retCnt = 0;

    if (rssme_invalidViInt16Range (action, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (StringLength (listName) > 7) {
        rssme_status = VI_ERROR_PARAMETER6;
        return rssme_status;
    }

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x200, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    if ((rssme_status = viPrintf (instrSession, ":SOUR:DM:DATA:CAT?")) < 0)
        return rssme_status;
    if ((rssme_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rssme_status;

    if (Scan (buffer, "%s>%s", oldCatalog) != 1) {
        rssme_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return rssme_status;
    }

    oldCatalog[retCnt - 1] = 0x0;

    if ((rssme_status = viPrintf (instrSession, ":SOUR:DM:DATA:FREE?")) < 0)
        return rssme_status;
    if ((rssme_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rssme_status;

    if (Scan (buffer, "%s>%i[b4],%i[b4]", oldVacantMem, oldOccupMem) != 2) {
        rssme_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return rssme_status;
    }

    if (action > 0) {
        switch (action) {
            case (1):
                if ((rssme_status = viPrintf (instrSession, ":SOUR:DM:DATA:SEL \"%s\";*WAI",
                        listName)) < 0)
                    return rssme_status;
            break;
            case (2):
                if ((rssme_status = viPrintf (instrSession, ":SOUR:DM:DATA:DEL \"%s\";*WAI",
                        listName)) < 0)
                    return rssme_status;
            break;
            case (3):
                if ((rssme_status = viPrintf (instrSession, ":SOUR:DM:DATA:DEL:ALL;*WAI")) < 0)
                    return rssme_status;
            break;
        }

        if ((rssme_status = viPrintf (instrSession, ":SOUR:DM:DATA:CAT?")) < 0)
            return rssme_status;
        if ((rssme_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
            return rssme_status;

        if (Scan (buffer, "%s>%s", newCatalog) != 1) {
            rssme_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            return rssme_status;
        }

        newCatalog[retCnt - 1] = 0x0;

        if ((rssme_status = viPrintf (instrSession, ":SOUR:DM:DATA:FREE?")) < 0)
            return rssme_status;
        if ((rssme_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
            return rssme_status;

        if (Scan (buffer, "%s>%i[b4],%i[b4]", newVacantMem, newOccupMem) != 2) {
            rssme_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            return rssme_status;
        }
    }
    else {
        Scan (oldCatalog, "%s>%s", newCatalog);
        *newVacantMem = *oldVacantMem;
        *newOccupMem = *oldOccupMem;
    }

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Create DM Data List                                             */
/* Purpose:  This function creates a new data list.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confDMCreateDataLst (ViSession instrSession, ViString listName,
    ViInt32 listLen, ViString data_arr, ViString atten_arr, ViString burst_arr)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;
    ViInt32 lenOfListLen = 0;

    if (StringLength (listName) > NAME_LEN) {
        rssme_status = VI_ERROR_PARAMETER2;
        return rssme_status;
    }
    if (rssme_invalidViInt32Range (listLen, 0, 1024))
        return VI_ERROR_PARAMETER3;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x200, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    if ((rssme_status = viPrintf (instrSession, ":SOUR:DM:DATA:SEL \"%s\";:FORM:DATA PACK;BORD SWAP",
            listName)) < 0)
        return rssme_status;

    Fmt (buffer, "%s<%i[b4]", listLen);
    lenOfListLen = NumFmtdBytes();

    Fmt (buffer, "%s<:SOUR:DM:DATA:DATA #%i[b2]%i[b4]" , lenOfListLen, listLen);
    bytesFmt = NumFmtdBytes ();

    Fmt (buffer, "%s[a]<%s[w*t-]", (int)listLen, data_arr);
    bytesFmt += NumFmtdBytes ();

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    Fmt (buffer, "%s<:SOUR:DM:DATA:ATT #%i[b2]%i[b4]" , lenOfListLen, listLen);
    bytesFmt = NumFmtdBytes ();

    Fmt (buffer, "%s[a]<%s[w*t-]", (int)listLen, atten_arr);
    bytesFmt += NumFmtdBytes ();

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    Fmt (buffer, "%s<:SOUR:DM:DATA:BURS #%i[b2]%i[b4]" , lenOfListLen, listLen);
    bytesFmt = NumFmtdBytes ();

    Fmt (buffer, "%s[a]<%s[w*t-]", (int)listLen, burst_arr);
    bytesFmt += NumFmtdBytes ();

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Upload DM Data List                                             */
/* Purpose:  This function uploads a data sequence into the instrument's DM  */
/*           extended memory.                                                */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confDMUpldList (ViSession instrSession, ViString fileName,
    ViString listName, ViInt16 dataMode, ViInt32 length)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViUInt32 lenOfListLen = 0;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;

    if (StringLength (listName) > NAME_LEN) {
        rssme_status = VI_ERROR_PARAMETER2;
        return rssme_status;
    }
    if (rssme_invalidViInt32Range (length, 1, 1024))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViInt16Range (dataMode, 0, 2))
        return VI_ERROR_PARAMETER4;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x200, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    Fmt (buffer, "%s<%i[b4]", length);
    lenOfListLen = NumFmtdBytes();

    Fmt (buffer, "%s<:SOUR:DM:DATA:SEL \"%s\";:FORM:DATA PACK;BORD SWAP", listName);
    bytesFmt = NumFmtdBytes ();

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, 0)) < 0)
        return rssme_status;

    Fmt (buffer, "%s<:SOUR:DM:DATA:%s #%i[b2]%i[b4]", dataTypeArr[dataMode], lenOfListLen, length);
    bytesFmt = NumFmtdBytes ();

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, 1)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_writeFromFile (instrSession, fileName, length, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Activate DM Data List                                           */
/* Purpose:  This function configures several parameters of DM (clock,       */
/*           attenuator) and activates data list.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confDMActiveLst (ViSession instrSession, ViString listName,
    ViInt16 PRBS, ViBoolean clkEdge, ViBoolean clkSource, ViBoolean clkMode,
    ViInt16 attenMode, ViReal64 attenLevel)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;

    if (StringLength (listName) > 7) {
        rssme_status = VI_ERROR_PARAMETER2;
        return rssme_status;
    }
    if (rssme_invalidViInt16Range (PRBS, 0, 4))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViBooleanRange (clkEdge))
        return VI_ERROR_PARAMETER4;
    if (rssme_invalidViBooleanRange (clkSource))
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViBooleanRange (clkMode))
        return VI_ERROR_PARAMETER6;
    if (rssme_invalidViInt16Range (attenMode, 0, 2))
        return VI_ERROR_PARAMETER7;
    if (rssme_invalidViReal64Range (attenLevel, 0.0, 60.0))
        return VI_ERROR_PARAMETER8;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x200, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    Scan (listName, "XMEM");
    if (NumFmtdBytes () == 4) {
        if ((rssme_status = rssme_queryInstrOption (instrSession, 0x400, &rssme_optPresent)) < 0)
            return rssme_status;
        if (rssme_optPresent == VI_OFF) {
            rssme_status = rssme_WARN_NUSP_OPTION;
            return rssme_status;
        }
    }

    if ((rssme_status = viPrintf (instrSession, ":SOUR:DM:PRBS %s;CLOC:POL %s;MODE %s;SOUR %s;:SOUR:DM:DATA:ALEV %f;%s;:SOUR:DM:DATA:SEL \"%s\"",
            PRBSArr[PRBS], polarityArr[clkEdge], clkModeArr[clkMode], clkSourceArr[clkSource],
            attenLevel, attenModeArr[attenMode], listName)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Upload Extended Memory                                          */
/* Purpose:  This function uploads a data sequence into the instrument's DM  */
/*           extended memory.                                                */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confDMUpldXMem (ViSession instrSession, ViString fileName,
    ViBoolean memMode, ViInt16 dataMode, ViInt32 startAddress, ViInt32 length)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViInt32 lenOfListLen = 0;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;

    if (rssme_invalidViBooleanRange (memMode))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViInt16Range (dataMode, 0, 2))
        return VI_ERROR_PARAMETER3;
    if (memMode) {
        if (rssme_invalidViInt32Range (startAddress, 1, 131070))
            return VI_ERROR_PARAMETER4;
        if (rssme_invalidViInt32Range (length, 3, 131070))
            return VI_ERROR_PARAMETER5;
    }
    else {
        if (rssme_invalidViInt32Range (startAddress, 1, 1048560))
            return VI_ERROR_PARAMETER4;
        if (rssme_invalidViInt32Range (length, 3, 1048560))
            return VI_ERROR_PARAMETER5;
    }

    if ((memMode == VI_ON) && (dataMode != 0)) {
        rssme_status = rssme_ERROR_DM_CONFLICT;
        return rssme_status;
    }

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x400, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    Fmt (buffer, "%s<%i[b4]", length);
    lenOfListLen = NumFmtdBytes();

    Fmt (buffer, "%s<:SOUR:DM:DATA:SEL \"XMEM\";:SOUR:DM:DATA:XMEM:STAR %i[b4];MODE %s;LENG:AUTO ON;:FORM:DATA PACK;BORD SWAP",
        startAddress, xmemModeArr[memMode]);
    bytesFmt = NumFmtdBytes ();

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, 0)) < 0)
        return rssme_status;

    Fmt (buffer, "%s<:SOUR:DM:DATA:%s #%i[b2]%i[b4]", dataTypeArr[dataMode], lenOfListLen, length);
    bytesFmt = NumFmtdBytes ();

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, 1)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_writeFromFile (instrSession, fileName, length, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Record Extended Memory From Ext Source                          */
/* Purpose:  This function configures the instrument to record data sequence */
/*           from an external generator.                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confDMRecordXMem (ViSession instrSession, ViInt32 startAddress,
    ViInt32 length, ViBoolean memMode, ViBoolean extTrig, ViBoolean trigSlope)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;

    if (rssme_invalidViBooleanRange (memMode))
        return VI_ERROR_PARAMETER4;
    if (memMode) {
        if (rssme_invalidViInt32Range (startAddress, 1, 1048560))
            return VI_ERROR_PARAMETER2;
        if (rssme_invalidViInt32Range (length, 3, 1048560))
            return VI_ERROR_PARAMETER3;
    }
    else {
        if (rssme_invalidViInt32Range (startAddress, 1, 8388480))
            return VI_ERROR_PARAMETER2;
        if (rssme_invalidViInt32Range (length, 3, 8388480))
            return VI_ERROR_PARAMETER3;
    }
    if (rssme_invalidViBooleanRange (extTrig))
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViBooleanRange (trigSlope))
        return VI_ERROR_PARAMETER6;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x400, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF) {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    Fmt (buffer, "%s<:SOUR:DM:DATA:XMEM:MODE %s;TRIG %s;:SOUR:DM:DATA:XMEM:TRIG:SLOP %s;:SOUR:DM:DATA:XMEM:STAR %i[b4];LENG %i[b4]",
        xmemModeArr[memMode], stateArr[extTrig], trigSlopeArr[trigSlope], startAddress, length);
    bytesFmt = NumFmtdBytes ();

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Frequency Sweep                                                 */
/* Purpose:  This function configures frequency sweep of the RF output.      */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confFreqSweep (ViSession instrSession, ViInt16 mode,
    ViInt16 freqInp, ViReal64 startFreq, ViReal64 stopFreq, ViReal64 centerFreq,
    ViReal64 spanFreq, ViReal64 dwell, ViReal64 step, ViBoolean spacing)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViReal64 rssme_maxFreq = 6E9;

    if (rssme_invalidViInt16Range (mode, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViInt16Range (freqInp, 0, 3))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViReal64Range (dwell, 1E-3, 1.0))
        return VI_ERROR_PARAMETER8;
    if (rssme_invalidViBooleanRange (spacing))
        return VI_ERROR_INSTR_PARAMETER10;
    if (spacing == VI_ON) {
        if (rssme_invalidViReal64Range (step, 0.01, 50.0))
            return VI_ERROR_INSTR_PARAMETER9;
    }
    else
        if (rssme_invalidViReal64Range (step, 0.0, 1E9))
            return VI_ERROR_INSTR_PARAMETER9;

    if ((rssme_status = rssme_queryInstrVersion (instrSession, &rssme_maxFreq)) < 0)
        return rssme_status;

    switch (freqInp) {
        case 1:
            startFreq = centerFreq - spanFreq / 2;
            stopFreq = centerFreq + spanFreq / 2;
            if (rssme_invalidViReal64Range (centerFreq, 5E3, rssme_maxFreq))
                return VI_ERROR_PARAMETER6;
            if (rssme_invalidViReal64Range (startFreq, 5E3, rssme_maxFreq))
                return VI_ERROR_PARAMETER7;
            if (rssme_invalidViReal64Range (stopFreq, 5E3, rssme_maxFreq))
                return VI_ERROR_PARAMETER7;
        break;
        case 2:
            stopFreq = startFreq + spanFreq;
            if (rssme_invalidViReal64Range (startFreq, 5E3, rssme_maxFreq))
                return VI_ERROR_PARAMETER4;
            if (rssme_invalidViReal64Range (stopFreq, 5E3, rssme_maxFreq))
                return VI_ERROR_PARAMETER7;
        break;
        case 3:
            startFreq = stopFreq - spanFreq;
            if (rssme_invalidViReal64Range (startFreq, 5E3, rssme_maxFreq))
                return VI_ERROR_PARAMETER5;
            if (rssme_invalidViReal64Range (stopFreq, 5E3, rssme_maxFreq))
                return VI_ERROR_PARAMETER7;
        break;
        default:
            if (rssme_invalidViReal64Range (startFreq, 5E3, rssme_maxFreq))
                return VI_ERROR_PARAMETER4;
            if (rssme_invalidViReal64Range (stopFreq, 5E3, rssme_maxFreq))
                return VI_ERROR_PARAMETER5;
        break;
    }

    if ((rssme_status = viPrintf (instrSession, ":SOUR:FREQ:STAR %f;STOP %f;:SOUR:SWE:DWEL %fS;:SOUR:SWE:FREQ:SPAC %s;STEP:%s %f;%s",
            startFreq, stopFreq, dwell, spacingArr[spacing], spacingArr[spacing], step,
            freqSweModeArr[mode])) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Frequency Sweep Markers                                         */
/* Purpose:  This function enables and configures frequency sweep markers.   */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confFreqSweepMrks (ViSession instrSession, ViReal64 freqM1,
    ViReal64 freqM2, ViReal64 freqM3, ViBoolean stateM1, ViBoolean stateM2,
    ViBoolean stateM3, ViBoolean amplM1, ViBoolean amplM2, ViBoolean amplM3, ViBoolean polarity)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViReal64 rssme_maxFreq = 6E9;

    if ((rssme_status = rssme_queryInstrVersion (instrSession, &rssme_maxFreq)) < 0)
        return rssme_status;

    if (rssme_invalidViReal64Range (freqM1, 5E3, rssme_maxFreq))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViReal64Range (freqM2, 5E3, rssme_maxFreq))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViReal64Range (freqM3, 5E3, rssme_maxFreq))
        return VI_ERROR_PARAMETER4;
    if (rssme_invalidViBooleanRange (stateM1))
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViBooleanRange (stateM2))
        return VI_ERROR_PARAMETER6;
    if (rssme_invalidViBooleanRange (stateM3))
        return VI_ERROR_PARAMETER7;
    if (rssme_invalidViBooleanRange (amplM1))
        return VI_ERROR_PARAMETER8;
    if (rssme_invalidViBooleanRange (amplM2))
        return VI_ERROR_INSTR_PARAMETER9;
    if (rssme_invalidViBooleanRange (amplM3))
        return VI_ERROR_INSTR_PARAMETER10;
    if (rssme_invalidViBooleanRange (polarity))
        return VI_ERROR_INSTR_PARAMETER11;

    if ((rssme_status = viPrintf (instrSession, ":SOUR:MARK1:AMPL %s;FREQ %f;STAT %s;:SOUR:MARK2:AMPL %s;FREQ %f;STAT %s;:SOUR:MARK3:AMPL %s;FREQ %f;STAT %s;:SOUR:MARK:POL %s",
            stateArr[amplM1], freqM1, stateArr[stateM1], stateArr[amplM2], freqM2, stateArr[stateM1],
            stateArr[amplM3], freqM3, stateArr[stateM3], polarityArr[polarity])) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Level Sweep                                                     */
/* Purpose:  This function configures level sweep of the RF output.          */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confLevelSweep (ViSession instrSession, ViInt16 mode,
    ViReal64 startLev, ViReal64 stopLev, ViInt16 unit, ViReal64 dwell, ViReal64 step)
{
    ViStatus rssme_status = VI_SUCCESS;

    if (rssme_invalidViInt16Range (mode, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViInt16Range (unit, 0, 7))
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViReal64Range (dwell, 1E-3, 1.0))
        return VI_ERROR_PARAMETER6;
    if (rssme_invalidViReal64Range (step, 0.1, 10.0))
        return VI_ERROR_PARAMETER7;

    if (unit == 0) {
        if (rssme_invalidViReal64Range (startLev, -144.0, 16.0))
            return VI_ERROR_PARAMETER3;
        if (rssme_invalidViReal64Range (stopLev, -144.0, 16.0))
            return VI_ERROR_PARAMETER4;
    }

    if ((rssme_status = viPrintf (instrSession, ":SOUR:POW:STAR %f%s;STOP %f%s;:SOUR:SWE:POW:DWEL %f;STEP:LOG %f;%s",
            startLev, unitArr[unit], stopLev, unitArr[unit], dwell, step, powSweModeArr[mode])) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Level Sweep Markers                                             */
/* Purpose:  This function enables and configures level sweep markers.       */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confLevelSweepMrks (ViSession instrSession, ViReal64 levelM1,
    ViReal64 levelM2, ViReal64 levelM3, ViBoolean stateM1, ViBoolean stateM2,
    ViBoolean stateM3, ViBoolean polarity)
{
    ViStatus rssme_status = VI_SUCCESS;

    if (rssme_invalidViReal64Range (levelM1, -144.0, 16.0))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViReal64Range (levelM2, -144.0, 16.0))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViReal64Range (levelM3, -144.0, 16.0))
        return VI_ERROR_PARAMETER4;
    if (rssme_invalidViBooleanRange (stateM1))
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViBooleanRange (stateM2))
        return VI_ERROR_PARAMETER6;
    if (rssme_invalidViBooleanRange (stateM3))
        return VI_ERROR_PARAMETER7;
    if (rssme_invalidViBooleanRange (polarity))
        return VI_ERROR_PARAMETER8;

    if ((rssme_status = viPrintf (instrSession, ":SOUR:MARK1:PSW:POW %f;STAT %s;:SOUR:MARK2:PSW:POW %f;STAT %s;:SOUR:MARK3:PSW:POW %f;STAT %s;:SOUR:MARK:POL %s",
            levelM1, stateArr[stateM1], levelM2, stateArr[stateM2], levelM3, stateArr[stateM3],
            polarityArr[polarity])) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*=========================================================================*/
/* Function: LF Generator                                                  */
/* Purpose:  This function selects the internal LF generator and configures*/
/*           its parameters.                                               */
/*=========================================================================*/
ViStatus _VI_FUNC rssme_confLFGen (ViSession instrSession, ViBoolean source,
    ViReal64 volt, ViReal64 freq1, ViInt16 shape1, ViReal64 freq2,  ViInt16 shape2,
    ViBoolean LF2mode)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViReal64 junk = 0.0;

    if (rssme_invalidViBooleanRange (source))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViReal64Range (volt, 0.0, 4.0))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViInt16Range (shape1, 0, 4))
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViInt16Range (shape2, 0, 4))
        return VI_ERROR_PARAMETER7;
    if (rssme_invalidViBooleanRange (LF2mode))
        return VI_ERROR_PARAMETER8;

    if (source == VI_ON) {
        if ((rssme_status = rssme_queryInstrOption (instrSession, 0x2, &rssme_optPresent)) < 0)
            return rssme_status;
        if (rssme_optPresent == VI_ON) {
            if (rssme_invalidViReal64Range (freq2, 0.1, 5E5))
                return VI_ERROR_PARAMETER6;
        }
        else {
            if ((rssme_status = rssme_queryInstrOption (instrSession, 0x40, &rssme_optPresent)) < 0)
                return rssme_status;
            if (rssme_optPresent == VI_ON) {
                if (rssme_invalidViReal64Range (freq2, 0.1, 1E6))
                    return VI_ERROR_PARAMETER6;
            }
            else {
                rssme_status = rssme_WARN_NUSP_OPTION;
                return rssme_status;
            }
        }
    if ((rssme_status = viPrintf (instrSession, ":SOUR2:FREQ:MODE %s;:SOUR2:FREQ:%s %f;:SOUR2:FUNC %s;:OUTP2:VOLT %f;SOUR 2",
            LFFreqModeArr[LF2mode], LFFreqInputModeArr[LF2mode], freq2, LFFunctionArr[shape2], volt)) < 0)
        return rssme_status;
    }

    else {
        if ((rssme_status = rssme_queryInstrOption (instrSession, 0x4, &rssme_optPresent)) < 0)
            return rssme_status;
        if (rssme_optPresent == VI_ON) {
            if (rssme_invalidViReal64Range (freq1, 0.1, 5E5))
                return VI_ERROR_PARAMETER4;
            if ((rssme_status = viPrintf (instrSession, ":SOUR0:FREQ %f;:SOUR0:FUNC %s;:OUTP2:VOLT %f;SOUR 0",
                    freq1, LFFunctionArr[shape1], volt)) < 0)
                return rssme_status;
        }
        else {
            if (rssme_optPresent == VI_FALSE) {
                switch ((ViInt16)freq1) {
                    case 400:
                    case 1000:
                    case 3000:
                    case 15000:
                        if (modf (freq1, &junk) != 0)
                            return VI_ERROR_PARAMETER4;
                        break;
                    default:
                        return VI_ERROR_PARAMETER4;
                }
                if (shape1 != 0)
                    return rssme_WARN_NUSP_OPTION;
            }
            if ((rssme_status = viPrintf (instrSession, ":SOUR0:FREQ %f;:OUTP2:SOUR 0", freq1)) < 0)
                return rssme_status;
        }
    }

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*=========================================================================*/
/* Function: LF Sweep                                                      */
/* Purpose:  This function configures frequency sweep of the internal      */
/*           LF generator.                                                 */
/*=========================================================================*/
ViStatus _VI_FUNC rssme_confLFSweep (ViSession instrSession, ViInt16 mode,
    ViReal64 startFreq, ViReal64 stopFreq, ViBoolean spacing, ViReal64 step,
    ViReal64 dwell)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;

    if (rssme_invalidViInt16Range (mode, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViReal64Range (startFreq, 0.1, 5E5))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViReal64Range (stopFreq, 0.1, 5E5))
        return VI_ERROR_PARAMETER4;
    if (rssme_invalidViBooleanRange (spacing))
        return VI_ERROR_PARAMETER5;
    if (spacing == VI_ON)   {
        if (rssme_invalidViReal64Range (step, 0.01, 50.0))
            return VI_ERROR_PARAMETER6;
    }
    else
        if (rssme_invalidViReal64Range (step, 0.0, 5E5))
            return VI_ERROR_PARAMETER6;
    if (rssme_invalidViReal64Range (dwell, 1E-3, 1.0))
        return VI_ERROR_PARAMETER7;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x2, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF)
        if ((rssme_status = rssme_queryInstrOption (instrSession, 0x40, &rssme_optPresent)) < 0)
            return rssme_status;

    if (rssme_optPresent == VI_ON) {
        if ((rssme_status = viPrintf (instrSession, ":SOUR2:FREQ:STAR %f;STOP %f;:SOUR2:SWE:DWEL %f;SPAC %s;STEP:%s %f;%s",
            startFreq, stopFreq, dwell, spacingArr[spacing], spacingArr[spacing], step, LFSweModeArr[mode])) < 0)
            return rssme_status;
    }
    else {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*=========================================================================*/
/* Function: LF Sweep Markers                                              */
/* Purpose:  This function enables and configures frequency sweep markers  */
/*           of the internal LF generator.                                 */
/*=========================================================================*/
ViStatus _VI_FUNC rssme_confLFSweepMrks (ViSession instrSession, ViReal64 freqM1,
    ViReal64 freqM2, ViReal64 freqM3, ViBoolean stateM1, ViBoolean stateM2,
    ViBoolean stateM3, ViBoolean polarity)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;

    if (rssme_invalidViReal64Range (freqM1, 0.1, 5E5))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViReal64Range (freqM2, 0.1, 5E5))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViReal64Range (freqM3, 0.1, 5E5))
        return VI_ERROR_PARAMETER4;
    if (rssme_invalidViBooleanRange (stateM1))
        return VI_ERROR_PARAMETER5;
    if (rssme_invalidViBooleanRange (stateM2))
        return VI_ERROR_PARAMETER6;
    if (rssme_invalidViBooleanRange (stateM3))
        return VI_ERROR_PARAMETER7;
    if (rssme_invalidViBooleanRange (polarity))
        return VI_ERROR_PARAMETER8;

    if ((rssme_status = rssme_queryInstrOption (instrSession, 0x2, &rssme_optPresent)) < 0)
        return rssme_status;
    if (rssme_optPresent == VI_OFF)
        if ((rssme_status = rssme_queryInstrOption (instrSession, 0x40, &rssme_optPresent)) < 0)
            return rssme_status;

    if (rssme_optPresent == VI_ON) {
        if ((rssme_status = viPrintf (instrSession, ":SOUR2:MARK1:FREQ %f;STAT %s;:SOUR2:MARK2:FREQ %f;STAT %s;:SOUR2:MARK3:FREQ %f;STAT %s;:SOUR2:MARK:POL %s",
            freqM1, stateArr[stateM1], freqM2, stateArr[stateM2], freqM3, stateArr[stateM3],
            polarityArr[polarity])) < 0)
            return rssme_status;
    }
    else {
        rssme_status = rssme_WARN_NUSP_OPTION;
        return rssme_status;
    }

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*=========================================================================*/
/* Function: Save/Recall Setup                                             */
/* Purpose:  This function saves and recalls instrument's setup.           */
/*=========================================================================*/
ViStatus _VI_FUNC rssme_confSaveRecallSetup (ViSession instrSession, ViInt32 stpNumber,
    ViBoolean setup, ViBoolean recallFreq, ViBoolean recallLevel)
{
    ViStatus rssme_status = VI_SUCCESS;

    if (rssme_invalidViBooleanRange (setup))
        return VI_ERROR_PARAMETER3;
    if (setup == VI_OFF) {
        if (rssme_invalidViInt32Range (stpNumber, 1, 50))
            return VI_ERROR_PARAMETER2;
    }
    else
        if (rssme_invalidViInt32Range (stpNumber, 0, 50))
            return VI_ERROR_PARAMETER2;
    if (rssme_invalidViBooleanRange (recallFreq))
        return VI_ERROR_PARAMETER4;
    if (rssme_invalidViBooleanRange (recallLevel))
        return VI_ERROR_PARAMETER5;

    if ((rssme_status = viPrintf (instrSession, ":SOUR:FREQ:RCL %s;:SOUR:POW:RCL %s;%s %ld", 
            recallArr[recallFreq], recallArr[recallLevel], stpArr[setup], stpNumber)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Edit Catalog                                                    */
/* Purpose:  This function allows the user to edit catalog of lists          */
/*           and memory sequences stored in the instrument's memory.         */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confEditCatalog (ViSession instrSession, ViInt16 action,
    ViChar _VI_FAR oldCatalog[], ViPInt32 oldOccupMem, ViPInt32 oldVacantMem, ViString listName,
    ViChar _VI_FAR newCatalog[], ViPInt32 newOccupMem, ViPInt32 newVacantMem, ViBoolean setup)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;

    if (rssme_invalidViInt16Range (action, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViBooleanRange (setup))
        return VI_ERROR_PARAMETER3;
    if (StringLength (listName) > 7) {
        rssme_status = VI_ERROR_PARAMETER4;
        return rssme_status;
    }

    if ((rssme_status = viPrintf (instrSession, "%s:CAT?", lstSeqArr[setup])) < 0)
        return rssme_status;
    if ((rssme_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rssme_status;

    if (Scan (buffer, "%s>%s", oldCatalog) != 1) {
        rssme_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return rssme_status;
    }

    oldCatalog[retCnt - 1] = 0x0;

    if ((rssme_status = viPrintf (instrSession, "%s:FREE?",lstSeqArr[setup])) < 0)
        return rssme_status;
    if ((rssme_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rssme_status;

    if (Scan (buffer, "%s>%i[b4],%i[b4]", oldVacantMem, oldOccupMem) != 2) {
        rssme_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return rssme_status;
    }

    if (action > 0) {
        switch (action) {
            case (1):
                if ((rssme_status = viPrintf (instrSession, "%s:SEL \"%s\";*WAI",
                    lstSeqArr[setup], listName)) < 0)
                    return rssme_status;
            break;
            case (2):
                if ((rssme_status = viPrintf (instrSession, "%s:DEL \"%s\";*WAI",
                    lstSeqArr[setup], listName)) < 0)
                    return rssme_status;
            break;
            case (3):
                if ((rssme_status = viPrintf (instrSession, ":SOUR:FREQ:MODE CW;*WAI;%s:DEL:ALL;*WAI",
                    lstSeqArr[setup])) < 0)
                    return rssme_status;
            break;
        }

        if ((rssme_status = viPrintf (instrSession, "%s:CAT?", lstSeqArr[setup])) < 0)
            return rssme_status;
        if ((rssme_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
            return rssme_status;

        if (Scan (buffer, "%s>%s", newCatalog) != 1) {
            rssme_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            return rssme_status;
        }

        newCatalog[retCnt - 1] = 0x0;

        if ((rssme_status = viPrintf (instrSession, "%s:FREE?", lstSeqArr[setup])) < 0)
            return rssme_status;
        if ((rssme_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
            return rssme_status;

        if (Scan (buffer, "%s>%i[b4],%i[b4]", newVacantMem, newOccupMem) != 2) {
            rssme_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            return rssme_status;
        }

    }
    else {
        Scan (oldCatalog, "%s>%s", newCatalog);
        *newVacantMem = *oldVacantMem;
        *newOccupMem = *oldOccupMem;
    }

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Create List                                                     */
/* Purpose:  This function creates a new list.                               */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confCreateLst (ViSession instrSession, ViString listName,
    ViInt32 listLen, ViReal64 _VI_FAR freq_arr[], ViReal64 _VI_FAR level_arr[], ViReal64 dwell)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;
    ViInt32 maxTmo = 15;
    ViInt16 i = 0;
    ViReal64 rssme_maxFreq = 6E9;

    if (StringLength (listName) > NAME_LEN) {
        rssme_status = VI_ERROR_PARAMETER2;
        return rssme_status;
    }
    if (rssme_invalidViInt32Range (listLen, 0, 2000))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViReal64Range (dwell, 1E-3, 1.0))
        return VI_ERROR_PARAMETER6;

    if ((rssme_status = rssme_queryInstrVersion (instrSession, &rssme_maxFreq)) < 0)
        return rssme_status;
    for (i=0; i<listLen; i++) {
        if (rssme_invalidViReal64Range (freq_arr[i], 5E3, rssme_maxFreq))
            return VI_ERROR_PARAMETER4;
        if (rssme_invalidViReal64Range (level_arr[i], -144.0, +16.0))
            return VI_ERROR_PARAMETER5;
    }

    if ((rssme_status = viPrintf (instrSession, ":SOUR:LIST:SEL \"%s\";:FORM:DATA REAL64;BORD SWAP",
        listName)) < 0)
        return rssme_status;

    Fmt (buffer, "%s<%i[b4]", listLen * BYTE_LEN);
    bytesFmt = NumFmtdBytes();

    Fmt (buffer, "%s<:SOUR:LIST:FREQ #%i[b2]%i[b4]" , bytesFmt, listLen * BYTE_LEN);
    bytesFmt = NumFmtdBytes ();

    Fmt (buffer, "%*f[zi*]<%*f", (ViInt16)listLen, bytesFmt, (ViInt16)listLen, freq_arr);
    bytesFmt += NumFmtdBytes ();

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    Fmt (buffer, "%s<%i[b4]", listLen * BYTE_LEN);
    bytesFmt = NumFmtdBytes();

    Fmt (buffer, "%s<:SOUR:LIST:POW #%i[b2]%i[b4]" , bytesFmt, listLen * BYTE_LEN);
    bytesFmt = NumFmtdBytes ();

    Fmt (buffer, "%*f[zi*]<%*f", (ViInt16)listLen, bytesFmt, (ViInt16)listLen, level_arr);
    bytesFmt += NumFmtdBytes ();

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = viPrintf (instrSession, ":SOUR:LIST:DWEL %f;*WAI", dwell)) < 0)
        return rssme_status;

    if ((rssme_status = viPrintf (instrSession, ":SOUR:LIST:LEAR")) < 0)
        return rssme_status;

    Delay (1.0);

    if ((rssme_operationComplete (instrSession, maxTmo))  < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*=========================================================================*/
/* Function: Create Memory Sequence                                        */
/* Purpose:  This function creates a new memory sequence.                  */
/*=========================================================================*/
ViStatus _VI_FUNC rssme_confCreateMemSeq (ViSession instrSession, ViString seqName,
    ViInt32 length, ViInt32 _VI_FAR stpList[], ViReal64 _VI_FAR dwellList[])
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt32 i = 0;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;

    if (rssme_invalidViInt32Range (length, 1, 256))
        return VI_ERROR_PARAMETER2;
    if (StringLength (seqName) > NAME_LEN) {
        rssme_status = VI_ERROR_PARAMETER3;
        return rssme_status;
    }

    if ((rssme_status = viPrintf (instrSession, ":SYST:MSEQ:SEL \"%s\"", seqName)) < 0)
        return rssme_status;

    Fmt (buffer, "%s<:SYST:MSEQ:RCL ");
    bytesFmt = NumFmtdBytes ();

    for (i = 0; i < length; i++) {
        if (rssme_invalidViInt32Range (stpList[i], 1, 256))
            return VI_ERROR_PARAMETER4;
        Fmt (buffer, "%s[a]<%i[b4],", stpList[i]);
        bytesFmt += NumFmtdBytes ();
    }

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt - 1, &retCnt)) < 0)
        return rssme_status;

    Fmt (buffer, "%s<:SYST:MSEQ:DWEL ");
    bytesFmt = NumFmtdBytes ();

    for (i = 0; i < length; i++) {
        if (rssme_invalidViReal64Range (dwellList[i], 0.05, 60.0))
            return VI_ERROR_PARAMETER5;
        Fmt (buffer, "%s[a]<%f[p3],", dwellList[i]);
        bytesFmt += NumFmtdBytes ();
    }

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt - 1, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*=========================================================================*/
/* Function: Select List/Sequence                                          */
/* Purpose:  This function activates a selected list or memory sequence.   */
/*=========================================================================*/
ViStatus _VI_FUNC rssme_confSelLstSeq (ViSession instrSession, ViInt16 setupMode,
    ViBoolean stepMode, ViString name)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt32 maxTmo = 15;

    if (rssme_invalidViInt16Range (setupMode, 0, 2))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViBooleanRange (stepMode))
        return VI_ERROR_PARAMETER3;
    if (StringLength (name) > NAME_LEN) {
        rssme_status = VI_ERROR_PARAMETER4;
        return rssme_status;
    }

    if (setupMode == 0) {
        if ((rssme_status = viPrintf (instrSession, ":SOUR:FREQ:MODE FIX;:SYST:MODE FIX")) < 0)
            return rssme_status;
    }
    else {
        if (setupMode == 1) {
            if ((rssme_status = viPrintf (instrSession, ":SYST:MODE FIX;:SOUR:FREQ:MODE FIX;:SOUR:LIST:SEL \"%s\"",
                name)) < 0)
                return rssme_status;
            if ((rssme_status = viPrintf (instrSession, ":SOUR:LIST:LEAR")) < 0)
                return rssme_status;
            if ((rssme_operationComplete (instrSession, maxTmo))  < 0)
                return rssme_status;
            if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
                return rssme_status;
            if ((rssme_status = viPrintf (instrSession, ":SOUR:LIST:MODE %s;:SOUR:FREQ:MODE LIST",
                 stepModeArr[stepMode])) < 0)
                return rssme_status;
        }
        else
            if ((rssme_status = viPrintf (instrSession, ":SOUR:FREQ:MODE FIX;:SYST:MODE MSEQ;:SYST:MSEQ:MODE %s;:SYST:MSEQ:SEL \"%s\"",
                stepModeArr[stepMode], name)) < 0)
                return rssme_status;
    }

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*=========================================================================*/
/* Function: Trigger                                                       */
/* Purpose:  This function configures the trigger system.                  */
/*=========================================================================*/
ViStatus _VI_FUNC rssme_confTrigger (ViSession instrSession, ViInt16 trigger,
    ViInt16 source, ViInt16 trigSlope)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 rssme_optPresent = VI_OFF;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;

    if (rssme_invalidViInt16Range (trigger, 0, 4))
        return VI_ERROR_PARAMETER2;
    if (trigger == 4) {
        if (rssme_invalidViInt16Range (source, 0, 1))
            return VI_ERROR_PARAMETER3;
        if (rssme_invalidViInt16Range (trigSlope, 0, 1))
            return VI_ERROR_PARAMETER4;
    }
    else {
        if (rssme_invalidViInt16Range (source, 0, 2))
            return VI_ERROR_PARAMETER3;
        if (rssme_invalidViInt16Range (trigSlope, 0, 2))
            return VI_ERROR_PARAMETER4;
    }

    switch (trigger) {
        case 0:
            Fmt (buffer, "%s<:TRIG1:SWE:SOUR %s;:TRIG1:SLOP %s", trigSourceArr[source], trigSlopeArr[trigSlope]);
            bytesFmt = NumFmtdBytes ();
            break;
        case 1:
            if ((rssme_status = rssme_queryInstrOption (instrSession, 0x2, &rssme_optPresent)) < 0)
                return rssme_status;
            if (rssme_optPresent == VI_OFF)
                if ((rssme_status = rssme_queryInstrOption (instrSession, 0x20, &rssme_optPresent)) < 0)
                    return rssme_status;
            if (rssme_optPresent == VI_ON) {
                Fmt (buffer, "%s<:TRIG2:SWE:SOUR %s;:TRIG2:SLOP %s", trigSourceArr[source], trigSlopeArr[trigSlope]);
                bytesFmt = NumFmtdBytes ();
            }
            else {
                rssme_status = rssme_WARN_NUSP_OPTION;
                return rssme_status;
            }
            break;
        case 2:
            Fmt (buffer, "%s<:TRIG:MSEQ:SOUR %s;:TRIG:SLOP %s", trigSourceArr[source], trigSlopeArr[trigSlope]);
            bytesFmt = NumFmtdBytes ();
            break;
        case 3:
            Fmt (buffer, "%s<:TRIG:LIST:SOUR %s;:TRIG:SLOP %s", trigSourceArr[source], trigSlopeArr[trigSlope]);
            bytesFmt = NumFmtdBytes ();
            break;
        case 4:
            if ((rssme_status = rssme_queryInstrOption (instrSession, 0x10, &rssme_optPresent)) < 0)
                return rssme_status;
            if (rssme_optPresent == VI_ON) {
                Fmt (buffer, "%s<:TRIG:PULS:SOUR %s;:TRIG:PULS:SLOP %s", trigSourceArr[source],
                    trigSlopeArr[trigSlope]);
                bytesFmt = NumFmtdBytes ();
            }
            else {
                rssme_status = rssme_WARN_NUSP_OPTION;
                return rssme_status;
            }
            break;
    }

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Instrument Options                                              */
/* Purpose:  This function returns the instrument version and list of        */
/*           options which the instrument is equipped with.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confOPTLst (ViSession instrSession, ViChar _VI_FAR optsList[], ViChar _VI_FAR instrVer[])
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViInt16 i = 0;

    FillBytes ((ViChar *)buffer, 0, 2048, '\0');

    if ((rssme_status = viWrite (instrSession, (ViBuf)"*IDN?", 5, &retCnt)) < 0)
        return rssme_status;
    if ((rssme_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rssme_status;

    for (i=0; i<3; i++) {
        if (FindPattern ((ViChar *)buffer, 0, -1, instrVersionList[i], 0, 0) >= 0) {
            Scan (instrVersionList[i], "%s>%s", instrVer);
            i = 9;
        }
    }
    if (i != 10) {
        rssme_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return rssme_status;
    }

    rssme_status = rssme_confOPT (instrSession, optsList);

    return rssme_status;
}

/*=========================================================================*/
/* Function: Reference Oscillator                                          */
/* Purpose:  This function selects and configures source of the reference  */
/*           frequency.                                                    */
/*=========================================================================*/
ViStatus _VI_FUNC rssme_confRefOscillator (ViSession instrSession,
    ViBoolean refSource, ViBoolean phasAdjust, ViBoolean unit, ViReal64 phase,
    ViBoolean freqAdjust, ViInt32 freqCorr)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;

    if (rssme_invalidViBooleanRange (refSource))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViBooleanRange (phasAdjust))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViBooleanRange (unit))
        return VI_ERROR_PARAMETER4;
    if (unit == VI_OFF) {
        if (rssme_invalidViReal64Range (phase, -360.0/180*PI, 360.0/180*PI))
            return VI_ERROR_PARAMETER5;
    }
    else
        if (rssme_invalidViReal64Range (phase, -360.0, 360.0))
            return VI_ERROR_PARAMETER5;
    if (rssme_invalidViBooleanRange (freqAdjust))
        return VI_ERROR_PARAMETER6;
    if (rssme_invalidViInt32Range (freqCorr, 0, 4095))
        return VI_ERROR_PARAMETER7;

    Fmt (buffer, "%s<:SOUR:ROSC:SOUR %s;:SOUR:ROSC:INT:ADJ:STAT %s;VAL %i[b4];:SOUR:PHAS:ADJ %f%s",
        sourceArr[refSource], stateArr[freqAdjust], freqCorr, phase,
        phaseUnitArr[unit]);
    bytesFmt = NumFmtdBytes ();

    if (phasAdjust == VI_ON) {
        Fmt (buffer, "%s[a]<;:SOUR:PHAS:REF");
        bytesFmt += NumFmtdBytes ();
    }

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*=========================================================================*/
/* Function: Outputs                                                       */
/* Purpose:  This function configures instrument's outputs.                */
/*=========================================================================*/
ViStatus _VI_FUNC rssme_confOuts (ViSession instrSession, ViBoolean blankPol,
    ViBoolean blankTime, ViBoolean protection, ViPInt32 trp)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;

    if (rssme_invalidViBooleanRange (blankPol))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViBooleanRange (blankTime))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViBooleanRange (protection))
        return VI_ERROR_PARAMETER4;

    if (protection == VI_ON) {
        if ((rssme_status = viPrintf (instrSession, ":OUTP:PROT:CLE;*WAI")) < 0)
            return rssme_status;
    }
    else {
        FillBytes ((ViChar *)buffer, 0, 1024, '\0');
        if ((rssme_status = viPrintf (instrSession, ":OUTP:PROT:TRIP?")) < 0)
            return rssme_status;
        if ((rssme_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
            return rssme_status;
        if (Fmt (trp, "%i[b4]<%s", buffer) < 0) {
            rssme_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            return rssme_status;
        }
    }

    if ((rssme_status = viPrintf (instrSession, ":OUTP:BLAN:POL %s;:SOUR:SWE:BTIM %s", polarityArr[blankPol],
        sweBlankTmArr[blankTime])) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Beeper                                                          */
/* Purpose:  This function activates and deactivates beeper on the instrument*/
/*===========================================================================*/
ViStatus _VI_FUNC rssme_confBeep (ViSession instrSession, ViBoolean beeper)
{
    ViStatus rssme_status = VI_SUCCESS;

    if (rssme_invalidViBooleanRange (beeper))
        return VI_ERROR_PARAMETER2;

    if ((rssme_status = viPrintf (instrSession, ":SYST:BEEP:STAT %s", stateArr[beeper])) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: RF Frequency                                                    */
/* Purpose:  This function sets the frequency of the RF output signal.       */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_qRFfrequency (ViSession instrSession,
                    ViReal64 freq, ViBoolean checkRange)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    ViReal64 rssme_maxFreq = 6E9;
    
    if (checkRange) {
        /*- Check input parameter ranges ------------------------------------*/
        if ((rssme_status = rssme_queryInstrVersion (instrSession, &rssme_maxFreq)) < 0)
            return rssme_status;
        if (rssme_invalidViReal64Range (freq, 5E3, rssme_maxFreq))
            return VI_ERROR_PARAMETER2;
    }
    Fmt (buffer, "%s<:FREQ %f[p4]HZ", freq);   
    if ((rssme_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssme_status;


    return rssme_status;
}

/*===========================================================================*/
/* Function: RF Level                                                        */
/* Purpose:  This function sets the level of the RF output signal.           */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_qRFlevel (ViSession instrSession,
                    ViReal64 level, ViBoolean checkRange)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    
    if (checkRange) {    
        /*- Check input parameter ranges ------------------------------------*/
        if (rssme_invalidViReal64Range (level, -144.0, 16.0))
            return VI_ERROR_PARAMETER2;
    }     

    Fmt (buffer, "%s<:POW %f[p2]DBM", level);   
    if ((rssme_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: RF Level Offset                                                 */
/* Purpose:  This function sets the level offset of the RF output signal.    */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_qRFlevOffset (ViSession instrSession,
                    ViReal64 offset, ViBoolean checkRange)

{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    
    if (checkRange) {    
        /*- Check input parameter ranges ------------------------------------*/
        if (rssme_invalidViReal64Range (offset, -100.0, 100.0))
            return VI_ERROR_PARAMETER2;
    }
    
    Fmt (buffer, "%s<:POW:OFFS %f[p2]", offset);   
    if ((rssme_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: RF Start Stop Frequency                                         */
/* Purpose:  This function sets the start and stop frequencies of the RF     */
/*           sweep.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_qRFstartStopFreq (ViSession instrSession,
                    ViReal64 startFreq, ViReal64 stopFreq, ViBoolean checkRange)                                  
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    ViReal64 rssme_maxFreq = 6E9;
    
    if (checkRange) {
        /*- Check input parameter ranges ------------------------------------*/
        if ((rssme_status = rssme_queryInstrVersion (instrSession, &rssme_maxFreq)) < 0)
            return rssme_status;
        if (rssme_invalidViReal64Range (startFreq, 5E3, rssme_maxFreq))
            return VI_ERROR_PARAMETER2;
        if (rssme_invalidViReal64Range (stopFreq, 5E3, rssme_maxFreq))
            return VI_ERROR_PARAMETER3;
    }

    Fmt (buffer, "%s<:FREQ:STAR %f[p2]HZ;STOP %f[p2]HZ", startFreq, stopFreq);
    if ((rssme_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: RF Center Span Frequency                                        */
/* Purpose:  This function sets the center and span frequencies of the RF    */
/*           sweep.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_qRFcenterSpanFreq (ViSession instrSession,
                    ViReal64 centerFreq, ViReal64 spanFreq, ViBoolean checkRange)                                  
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    ViReal64 rssme_maxFreq = 6E9;
    
    if (checkRange) {
        /*- Check input parameter ranges ------------------------------------*/
        if ((rssme_status = rssme_queryInstrVersion (instrSession, &rssme_maxFreq)) < 0)
            return rssme_status;
        if (rssme_invalidViReal64Range (centerFreq, 5E3, rssme_maxFreq))
            return VI_ERROR_PARAMETER2;
        if (rssme_invalidViReal64Range (spanFreq, 5E3, rssme_maxFreq))
            return VI_ERROR_PARAMETER3;
    }
    
    Fmt (buffer, "%s<:FREQ:CENT %f[p2]HZ;SPAN %f[p2]HZ", centerFreq, spanFreq);
    if ((rssme_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssme_status;
    
    return rssme_status;
}

/*===========================================================================*/
/* Function: RF Sweep Step                                                   */
/* Purpose:  This function sets the RF sweep step in MHz.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_qRFsweepStep (ViSession instrSession,
                    ViReal64 step, ViBoolean checkRange)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];

    if (checkRange) {
        /*- Check input parameter ranges ------------------------------------*/     
        if (rssme_invalidViReal64Range (step, 0.0, 1.0E9))
            return VI_ERROR_PARAMETER2;
    }

    Fmt (buffer, "%s<:SWE:STEP %f[p2]HZ", step);
    if ((rssme_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssme_status;
    
    return rssme_status;
}
                                  
/*===========================================================================*/
/* Function: LF Frequency                                                    */
/* Purpose:  This function sets the LF output frequency.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_qLFfreq (ViSession instrSession, ViBoolean source,
                    ViReal64 freq, ViBoolean checkRange)                           
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    ViInt16 rssme_optPresent = VI_OFF;
    ViReal64 junk = 0.0;
    
    if (checkRange) {    
        /*- Check input parameter ranges ------------------------------------*/
        if (rssme_invalidViBooleanRange (source))        
            return VI_ERROR_PARAMETER2;
        if (rssme_invalidViReal64Range (freq, 0.1, 1E6))
            return VI_ERROR_PARAMETER3;
        
        if (source == VI_ON) {  /* LF Gen 2 */
            if ((rssme_status = rssme_queryInstrOption (instrSession, 0x2, &rssme_optPresent)) < 0)
                return rssme_status;
            if (rssme_optPresent == VI_ON) {
                if (rssme_invalidViReal64Range (freq, 0.1, 5E5))
                    return VI_ERROR_PARAMETER3;
            }
            else {
                if ((rssme_status = rssme_queryInstrOption (instrSession, 0x40, &rssme_optPresent)) < 0)
                    return rssme_status;
                if (rssme_optPresent == VI_ON) {
                    if (rssme_invalidViReal64Range (freq, 0.1, 1E6))
                        return VI_ERROR_PARAMETER3;
                }
                else {
                    rssme_status = rssme_WARN_NUSP_OPTION;
                    return rssme_status;
                }
            }
        }

        else {                  /* LF Gen 1 */
            if ((rssme_status = rssme_queryInstrOption (instrSession, 0x4, &rssme_optPresent)) < 0)
                return rssme_status;
            if (rssme_optPresent == VI_ON) {
                if (rssme_invalidViReal64Range (freq, 0.1, 5E5))
                    return VI_ERROR_PARAMETER3;
            }
            else {
                if (rssme_optPresent == VI_FALSE) {
                    switch ((ViInt16)freq) {
                        case 400:
                        case 1000:
                        case 3000:
                        case 15000:
                            if (modf (freq, &junk) != 0)
                                return VI_ERROR_PARAMETER3;
                            break;
                        default:
                            return VI_ERROR_PARAMETER3;
                    }
                }
            }
        }
    }
    
    Fmt (buffer, ":SOUR%s:FREQ %f[p2]HZ", zeroOrTwo[source], freq);
    if ((rssme_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssme_status;
    
    return rssme_status;
}

/*===========================================================================*/
/* Function: LF Voltage                                                      */
/* Purpose:  This function sets the LF output voltage.                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_qLFvolt (ViSession instrSession, ViReal64 volt,
                    ViBoolean checkRange)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViInt16 rssme_optPresent = VI_OFF;
    ViByte buffer[BUFFER_SIZE];
    
    if (checkRange) {   
        /*- Check input parameter ranges ------------------------------------*/
        if ((rssme_status = rssme_queryInstrOption (instrSession, 0x2, &rssme_optPresent)) < 0)
            return rssme_status;
        if (rssme_optPresent != VI_ON) {
            if ((rssme_status = rssme_queryInstrOption (instrSession, 0x40, &rssme_optPresent)) < 0)
                return rssme_status;
            if (rssme_optPresent != VI_ON) {
                rssme_status = rssme_WARN_NUSP_OPTION;
                return rssme_status;
            }
        }
        if (rssme_invalidViReal64Range (volt, 0.0, 4.0))        
            return VI_ERROR_PARAMETER2;        
    }
    
    Fmt (buffer, "%s<:OUTP2:VOLT %f[p3]", volt);
    if ((rssme_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssme_status;
    
    return rssme_status;
}
                         
/*===========================================================================*/
/* Function: AM Modulation Depth                                             */
/* Purpose:  This function sets the AM modulation depth.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_qAMmodDep (ViSession instrSession,
                    ViReal64 modDepth, ViBoolean checkRange)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    
    if (checkRange) {    
        /*- Check input parameter ranges ------------------------------------*/
        if (rssme_invalidViReal64Range (modDepth, 0.0, 100.0))
            return VI_ERROR_PARAMETER2;
    }
    
    Fmt (buffer, "%s<:AM %f[p2]", modDepth);
    if ((rssme_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: FM Deviation                                                    */
/* Purpose:  This function sets the FM Deviation.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_qFMdev (ViSession instrSession, ViBoolean modulator,
                    ViReal64 freqDev, ViBoolean checkRange)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    ViReal64 rssme_maxFreq = 6E9;

    if (checkRange) {    
        /*- Check input parameter ranges ------------------------------------*/
        if (rssme_invalidViBooleanRange (modulator))
            return VI_ERROR_PARAMETER2;
        if ((rssme_status = rssme_queryInstrVersion (instrSession, &rssme_maxFreq)) < 0)
            return rssme_status;
        if (rssme_maxFreq == 1.5E9) {  /* SME02 */
            if (rssme_invalidViReal64Range (freqDev, 0.0, 1E6))
                return VI_ERROR_PARAMETER3;
        }           
        if (rssme_maxFreq == 3E9) {    /* SME03 */
            if (rssme_invalidViReal64Range (freqDev, 0.0, 2E6))
                return VI_ERROR_PARAMETER3;
        }       
        if (rssme_maxFreq == 6E9) {    /* SME06 */
            if (rssme_invalidViReal64Range (freqDev, 0.0, 4E6))
                return VI_ERROR_PARAMETER3;
        }           
    }
    
    Fmt (buffer, "%s<:FM%s %f[p2]HZ", oneOrTwo[modulator], freqDev);

    if ((rssme_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssme_status;
    
    return rssme_status;
}

/*===========================================================================*/
/* Function: Phase Mod Deviation                                             */
/* Purpose:  This function sets the PHM Deviation.                           */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_qPHMdev (ViSession instrSession, ViBoolean modulator,
                    ViReal64 phaseDev, ViBoolean checkRange)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    
    if (checkRange) {    
        /*- Check input parameter ranges ------------------------------------*/
        if (rssme_invalidViBooleanRange (modulator))
            return VI_ERROR_PARAMETER2;
        if (rssme_invalidViReal64Range (phaseDev, -2*PI, 2*PI))
            return VI_ERROR_PARAMETER3;
    }
    
    Fmt (buffer, "%s<:PM%s %f[p3]RAD", oneOrTwo[modulator], phaseDev);

    if ((rssme_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssme_status;
    
    return rssme_status;
}

/*===========================================================================*/
/* Function: Pulse Modulator Delay                                           */
/* Purpose:  This function sets the pulse modulator delay.                   */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_qPulsModDelay (ViSession instrSession, ViReal64 delay,
                    ViBoolean checkRange)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    ViInt16 rssme_optPresent = VI_OFF;
    
    if (checkRange) {    
        /*- Check input parameter ranges ------------------------------------*/ 
        if ((rssme_status = rssme_queryInstrOption (instrSession, 0x10, &rssme_optPresent)) < 0)
            return rssme_status;     
        if (rssme_optPresent == VI_OFF) {
            rssme_status = rssme_WARN_NUSP_OPTION;
            return rssme_status;
        }
        if (rssme_invalidViReal64Range (delay, 40.0E-9, 1.0))
            return VI_ERROR_PARAMETER2;
    }
    
    Fmt (buffer, "%s<:PULS:DEL %fS", delay);
    if ((rssme_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssme_status;
    
    return rssme_status;
}

/*===========================================================================*/
/* Function: Pulse Modulator Period                                          */
/* Purpose:  This function sets the pulse modulator period.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_qPulsModPeriod (ViSession instrSession, ViReal64 period,
                    ViBoolean checkRange)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    ViInt16 rssme_optPresent = VI_OFF;
    
    if (checkRange) {    
        /*- Check input parameter ranges ------------------------------------*/ 
        if ((rssme_status = rssme_queryInstrOption (instrSession, 0x10, &rssme_optPresent)) < 0)
            return rssme_status;     
        if (rssme_optPresent == VI_OFF) {
            rssme_status = rssme_WARN_NUSP_OPTION;
            return rssme_status;
        }
        if (rssme_invalidViReal64Range (period, 100.0E-9, 85.0))
            return VI_ERROR_PARAMETER2;
    }
    
    Fmt (buffer, "%s<:PULS:PER %fS", period);
    if ((rssme_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssme_status;
    
    return rssme_status;
}

/*===========================================================================*/
/* Function: Pulse Modulator Width                                           */
/* Purpose:  This function sets the pulse modulator width.                   */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_qPulsModWidth (ViSession instrSession, ViReal64 width,
                    ViBoolean checkRange)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    ViInt16 rssme_optPresent = VI_OFF;
    
    if (checkRange) {    
        /*- Check input parameter ranges ------------------------------------*/ 
        if ((rssme_status = rssme_queryInstrOption (instrSession, 0x10, &rssme_optPresent)) < 0)
            return rssme_status;     
        if (rssme_optPresent == VI_OFF) {
            rssme_status = rssme_WARN_NUSP_OPTION;
            return rssme_status;
        }
        if (rssme_invalidViReal64Range (width, 20.0E-9, 1.0))
            return VI_ERROR_PARAMETER2;
    }
    
    Fmt (buffer, "%s<:PULS:WIDT %fS", width);
    if ((rssme_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssme_status;
    
    return rssme_status;
}

/*===========================================================================*/
/* Function: Stereo Modulation Deviation                                     */
/* Purpose:  This function sets stereo modulation deviation.                 */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_qStereoModDev (ViSession instrSession, ViReal64 deviation,
                    ViBoolean checkRange)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    ViInt16 rssme_optPresent = VI_OFF;
    
    if (checkRange) {    
        /*- Check input parameter ranges ------------------------------------*/ 
        if ((rssme_status = rssme_queryInstrOption (instrSession, 0x20, &rssme_optPresent)) < 0)
            return rssme_status;     
        if (rssme_optPresent == VI_OFF) {
            if ((rssme_status = rssme_queryInstrOption (instrSession, 0x40, &rssme_optPresent)) < 0)
                return rssme_status;     
            if (rssme_optPresent == VI_OFF) {
                rssme_status = rssme_WARN_NUSP_OPTION;
                return rssme_status;
            }               
        }
        if (rssme_invalidViReal64Range (deviation, 0.0, 100.0E3))
            return VI_ERROR_PARAMETER2;
    }
    
    Fmt (buffer, "%s<:STER %fHZ", deviation);
    if ((rssme_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssme_status;
    
    return rssme_status;
}

/*===========================================================================*/
/* Function: Stereo Modulation Audio Frequency                               */
/* Purpose:  This function sets stereo modulation audio frequency.           */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_qStereoModAudioFreq (ViSession instrSession, ViReal64 audioFreq,
                    ViBoolean checkRange)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    ViInt16 rssme_optPresent = VI_OFF;
    
    if (checkRange) {    
        /*- Check input parameter ranges ------------------------------------*/ 
        if ((rssme_status = rssme_queryInstrOption (instrSession, 0x20, &rssme_optPresent)) < 0)
            return rssme_status;     
        if (rssme_optPresent == VI_OFF) {
            if ((rssme_status = rssme_queryInstrOption (instrSession, 0x40, &rssme_optPresent)) < 0)
                return rssme_status;     
            if (rssme_optPresent == VI_OFF) {
                rssme_status = rssme_WARN_NUSP_OPTION;
                return rssme_status;
            }               
        }
        if (rssme_invalidViReal64Range (audioFreq, 0.1, 15E3))
            return VI_ERROR_PARAMETER2;
    }
    
    Fmt (buffer, "%s<:STER:AUD %f[p2]HZ", audioFreq);
    if ((rssme_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rssme_status;
    
    return rssme_status;
}

/*=========================================================================*/
/* Function: Outputs On/Off                                                */
/* Purpose:  This function turns instrument's outputs on or off.           */
/*=========================================================================*/
ViStatus _VI_FUNC rssme_outState (ViSession instrSession, ViBoolean RFstate,
    ViBoolean LFstate)
{
    ViStatus rssme_status = VI_SUCCESS;

    if (rssme_invalidViBooleanRange (RFstate))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViBooleanRange (LFstate))
        return VI_ERROR_PARAMETER3;

    if ((rssme_status = viPrintf (instrSession, ":OUTP1 %s;:OUTP2 %s",
        stateArr[RFstate], stateArr[LFstate])) < 0)
        return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*=========================================================================*/
/* Function: Send Trigger                                                  */
/* Purpose:  This function triggers the instrument.                        */
/*=========================================================================*/
ViStatus _VI_FUNC rssme_sendTrig (ViSession instrSession, ViInt16 trigger)
{
    ViStatus rssme_status = VI_SUCCESS;

    if (rssme_invalidViInt16Range (trigger, 0, 3))
        return VI_ERROR_PARAMETER2;

    if ((rssme_status = viPrintf (instrSession, ":TRIG:%s:IMM", trigAbortArr[trigger])) < 0)
        return rssme_status;

    return rssme_status;
}

/*=========================================================================*/
/* Function: Abort                                                         */
/* Purpose:  This function aborts an operation which is in progress.       */
/*=========================================================================*/
ViStatus _VI_FUNC rssme_abort (ViSession instrSession, ViInt16 trigger)
{
    ViStatus rssme_status = VI_SUCCESS;

    if (rssme_invalidViInt16Range (trigger, 0, 3))
        return VI_ERROR_PARAMETER2;

    if ((rssme_status = viPrintf (instrSession, ":ABOR:%s", trigAbortArr[trigger])) < 0)
        return rssme_status;

    return rssme_status;
}

/*=========================================================================*/
/* Function: Record Extended Memory                                        */
/* Purpose:  This function starts the recording of external data into      */
/*           the DM memory extention.                                      */
/*=========================================================================*/
ViStatus _VI_FUNC rssme_recordXMem (ViSession instrSession)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;

    Fmt (buffer, "%s<:SOUR:DM:DATA:XMEM:REC");
    bytesFmt = NumFmtdBytes ();

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    return rssme_status;
}

/*=========================================================================*/
/* Function: Abort Recording Extended Memory                               */
/* Purpose:  This function aborts recording of data into the DM memory     */
/*           extention.                                                    */
/*=========================================================================*/
ViStatus _VI_FUNC rssme_abortXMem (ViSession instrSession)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;

    Fmt (buffer, "%s<:ABOR:XMEM");
    bytesFmt = NumFmtdBytes ();

    if ((rssme_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rssme_status;

    return rssme_status;
}

/*=========================================================================*/
/* Function: Recalculate Digital Modulation                                */
/* Purpose:  This function recalculates parameters of a digital modulation.*/
/*=========================================================================*/
ViStatus _VI_FUNC rssme_recalculDM (ViSession instrSession, ViInt16 modulType)
{
    ViStatus rssme_status = VI_SUCCESS;

    if (rssme_invalidViInt16Range (modulType, 0, 2))
        return VI_ERROR_PARAMETER2;

    if ((rssme_status = viPrintf (instrSession, ":SOUR:%s:STAT OFF;STAT ON", DMRecalculArr[modulType])) < 0)
        return rssme_status;

    return rssme_status;
}

/*=========================================================================*/
/* Function: Step                                                          */
/* Purpose:  This function configures and performes step up/down of a      */
/*           specified parameter.                                          */
/*=========================================================================*/
ViStatus _VI_FUNC rssme_step (ViSession instrSession, ViBoolean mode,
    ViBoolean direction, ViInt16 parameter, ViReal64 stepSize)
{
    ViStatus rssme_status = VI_SUCCESS;

    if (rssme_invalidViBooleanRange (mode))
        return VI_ERROR_PARAMETER2;
    if (rssme_invalidViBooleanRange (direction))
        return VI_ERROR_PARAMETER3;
    if (rssme_invalidViInt16Range (parameter, 0, 3))
        return VI_ERROR_PARAMETER4;
    if (parameter > 2) {
        if (rssme_invalidViReal64Range (stepSize, 0.1, 10))
            return VI_ERROR_PARAMETER5;
    }
    else
        if (rssme_invalidViReal64Range (stepSize, 0.0, 1E9))
            return VI_ERROR_PARAMETER5;

    if (mode == VI_ON) {
        if ((rssme_status = viPrintf (instrSession, "%s %f", stepConfigArr[parameter], stepSize)) < 0)
            return rssme_status;
    }
    else
        if ((rssme_status = viPrintf (instrSession, "%s %s", stepExeArr[parameter], directArr[direction])) < 0)
            return rssme_status;

    if ((rssme_status = rssme_sysStatus (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*=========================================================================*/
/* Function: Operation Complete                                            */
/* Purpose:  This function waits until all previous operations will have   */
/*           been completed.                                               */
/*=========================================================================*/
ViStatus _VI_FUNC rssme_operationComplete (ViSession instrSession, ViInt32 maxTimeout)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViUInt16 response = 0;
    ViInt32 i = 0;

    if (rssme_invalidViReal64Range (maxTimeout, 0.0, 600))
        return VI_ERROR_PARAMETER2;

    if ((rssme_status = viWrite (instrSession, (ViBuf)"*WAI;*OPC?", 10, &retCnt)) < 0)
        return rssme_status;

    while (!(response & 0x10)) {

        if ((rssme_status = viReadSTB (instrSession, &response)) < 0)
            return rssme_status;

        Delay(0.1);
        i++;

        if (i/10 > maxTimeout) {
            rssme_status = VI_ERROR_TMO;
            return rssme_status;
        }
    }

    if ((rssme_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Write To Instrument                                             */
/* Purpose:  This function writes a command string to the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_writeInstrData (ViSession instrSession, ViString writebuffer)
{
    ViStatus rssme_status = VI_SUCCESS;

    if ((rssme_status = viPrintf (instrSession, "%s", writebuffer)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Read Instrument buffer                                          */
/* Purpose:  This function reads the output buffer of the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_readInstrData (ViSession instrSession, ViInt16 numBytes,
                    ViChar _VI_FAR rdBuf[], ViPInt32 bytesRead)
{
    ViStatus rssme_status = VI_SUCCESS;
	ViUInt32 tmpVal;

	if (bytesRead)
		*bytesRead = 0L;

    if ((rssme_status = viRead (instrSession, (ViBuf)rdBuf, numBytes, &tmpVal)) < 0)
        return rssme_status;
	
	if (bytesRead)
		*bytesRead = (ViInt32)tmpVal;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_reset (ViSession instrSession)
{
    ViUInt32 retCnt = 0;
    ViStatus rssme_status = VI_SUCCESS;

    /*  Initialize the instrument to a known state.  */
    if ((rssme_status = viWrite (instrSession, (ViBuf)"*RST", 4, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = rssme_defaultInstrSetup (instrSession)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Self-Test                                                       */
/* Purpose:  This function executes the instrument self-test and returns     */
/*           the result.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_self_test (ViSession instrSession, ViPInt16 result,
                                   ViChar _VI_FAR message[])
{
    ViUInt32 retCnt = 0;
    ViStatus rssme_status = VI_SUCCESS;
	ViInt16	 tmpVal;

    *result = 0;

    if ((rssme_status = viWrite (instrSession, (ViBuf)":TEST:RAM?", 10, &retCnt)) < 0)
        return rssme_status;
    if ((rssme_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rssme_status;

	if (Scan (buffer, "%s>%i[b2]", &tmpVal) < 0)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	if (tmpVal != 0)
		{
		*result = tmpVal;
		strcpy(message, "RAM test error.");
		return rssme_status;
		}

    if ((rssme_status = viWrite (instrSession, (ViBuf)":TEST:ROM?", 10, &retCnt)) < 0)
        return rssme_status;
    if ((rssme_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rssme_status;

    if (Scan (buffer, "%s>%i[b2]", &tmpVal) < 0)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if (tmpVal != 0)
		{
		*result = tmpVal;
		strcpy(message, "ROM test error.");
		return rssme_status;
		}

    if ((rssme_status = viWrite (instrSession, (ViBuf)":TEST:BATT?", 11, &retCnt)) < 0)
        return rssme_status;
    if ((rssme_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rssme_status;

	if (Scan (buffer, "%s>%i[b2]", &tmpVal) < 0)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if (tmpVal != 0)
		{
		*result = tmpVal;
		strcpy(message, "Battery test error.");
		return rssme_status;
		}

    return rssme_status;
}

/*===========================================================================*/
/* Function: Error Query                                                     */
/* Purpose:  This function queries the instrument error queue, and returns   */
/*           the result.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_error_query (ViSession instrSession, ViPInt32 errCode,
                    ViChar _VI_FAR errMessage[])
{
    ViUInt32 retCnt = 0;
    ViStatus rssme_status = VI_SUCCESS;

    if ((rssme_status = viWrite (instrSession, (ViBuf)":SYST:ERR?", 10, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = viScanf (instrSession, "%ld,\"%[^\"]", errCode, errMessage)) < 0)
        return rssme_status;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_error_message (ViSession instrSession, ViStatus errorCode,
                    ViChar _VI_FAR errMessage[])
{
    ViStatus rssme_status = VI_SUCCESS;
    ViInt16 i;
    static rssme_tStringValPair statusDescArray[] = {
        {VI_WARN_NSUP_ID_QUERY,     "WARNING: ID Query not supported"},
        {VI_WARN_NSUP_RESET,        "WARNING: Reset not supported"},
        {VI_WARN_NSUP_SELF_TEST,    "WARNING: Self-test not supported"},
        {VI_WARN_NSUP_ERROR_QUERY,  "WARNING: Error Query not supported"},
        {VI_WARN_NSUP_REV_QUERY,    "WARNING: Revision Query not supported"},
        {VI_ERROR_PARAMETER1,   "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,   "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,   "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,   "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,   "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,   "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,   "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,   "ERROR: Parameter 8 out of range"},
        {VI_ERROR_INSTR_PARAMETER9,   "ERROR: Parameter 9 out of range"},
        {VI_ERROR_INSTR_PARAMETER10,  "ERROR: Parameter 10 out of range"},
        {VI_ERROR_INSTR_PARAMETER11,  "ERROR: Parameter 11 out of range"},
        {VI_ERROR_INSTR_PARAMETER12,  "ERROR: Parameter 12 out of range"},
        {VI_ERROR_INSTR_PARAMETER13,  "ERROR: Parameter 13 out of range"},
        {VI_ERROR_INSTR_PARAMETER14,  "ERROR: Parameter 14 out of range"},
        {VI_ERROR_INSTR_PARAMETER15,  "ERROR: Parameter 15 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,"ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE, "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_FILE_OPEN,    "ERROR: Opening the specified file"},
        {VI_ERROR_INSTR_FILE_WRITE,   "ERROR: Writing to the specified file"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        {rssme_WARN_NUSP_OPTION,   "WARNING: Not supported instrument option"},
        {rssme_ERROR_DM_CONFLICT,  "ERROR: Memory mode vs. data type conflict"},
        {rssme_ERROR_INSTRUMENT_ERROR,  "ERROR: Instrument specific error"},
        {VI_NULL, VI_NULL}
    };

    rssme_status = viStatusDesc (instrSession, errorCode, errMessage);
    if (rssme_status == VI_WARN_UNKNOWN_STATUS) {
        for (i=0; statusDescArray[i].stringName; i++) {
            if (statusDescArray[i].stringVal == errorCode) {
                Fmt (errMessage, "%s<%s", statusDescArray[i].stringName);
                return (VI_SUCCESS);
            }
        }
        Fmt (errMessage, "%s<Unknown Error 0x%x[uw8p0]", errorCode);
        return (VI_WARN_UNKNOWN_STATUS);
    }

    rssme_status = VI_SUCCESS;
    return rssme_status;
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_revision_query (ViSession instrSession, ViChar _VI_FAR driverRev[],
    ViChar _VI_FAR instrRev[])
{
    ViUInt32 retCnt = 0;
    ViStatus rssme_status = VI_SUCCESS;

    if ((rssme_status = viWrite (instrSession, (ViBuf)"*IDN?", 5, &retCnt)) < 0)
        return rssme_status;

    if ((rssme_status = viScanf (instrSession, "%*[^,],%*[^,],%*[^,],%[^\n]", instrRev)) < 0)
        return rssme_status;

    Fmt (driverRev, "%s<%s", RSSME_REVISION);

    return rssme_status;
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssme_close (ViSession instrSession)
{
    rssme_instrRange instrPtr;
    ViSession rmSession;
    ViStatus rssme_status = VI_SUCCESS;

    if ((rssme_status = viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession)) < 0)
        return rssme_status;
    if ((rssme_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssme_status;

    if (instrPtr != NULL) 
        free (instrPtr);

    rssme_status = viClose (instrSession);
    viClose (rmSession);

    return rssme_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_ON   */
/*           or VI_OFF. If the value is out of range, the return value is    */
/*           VI_ON, otherwise the return value is VI_OFF.                    */
/*===========================================================================*/
ViBoolean rssme_invalidViBooleanRange (ViBoolean val)
{
    return (ViBoolean)((val != VI_OFF && val != VI_ON) ? VI_ON : VI_OFF);
}

/*===========================================================================*/
/* Function: Short Signed Integer Value Out Of Range - ViInt16               */
/* Purpose:  This function checks a short signed integer value to see if it  */
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_ON, otherwise the return       */
/*           value is VI_OFF.                                                */
/*===========================================================================*/
ViBoolean rssme_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_ON : VI_OFF);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range,  the return value is VI_ON, otherwise the return      */
/*           value is VI_OFF.                                                */
/*===========================================================================*/
ViBoolean rssme_invalidViInt32Range  (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_ON : VI_OFF);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_ON, otherwise the return value    */
/*           is VI_OFF.                                                      */
/*===========================================================================*/
ViBoolean rssme_invalidViReal64Range  (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_ON : VI_OFF);
}

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the rssme_init function.  When    */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from rssme_init to        */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus rssme_initCleanUp (ViSession openRMSession,
                    ViPSession openInstrSession, ViStatus currentStatus)
{
    rssme_instrRange instrPtr;
    
    if (viGetAttribute (*openInstrSession, VI_ATTR_USER_DATA, &instrPtr) >= 0)
        if (instrPtr != NULL) 
            free (instrPtr);
   
    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;

    return currentStatus;
}

/*===========================================================================*/
/* Function: Write From File To Instrument                                   */
/* Purpose:  This function is used to read data from a user specified file   */
/*           and write it to the instrument.                                 */
/*===========================================================================*/
ViStatus rssme_writeFromFile (ViSession instrSession, ViChar filename[],
                    ViUInt32 writeBytes, ViPUInt32 retCount)
{
    ViStatus  rssme_status = VI_SUCCESS;
    ViByte    buffer[BUFFER_SIZE];
    ViUInt32  bytesRead = 0, bytesWritten = 0;
    FILE     *sourceFile;
    ViBoolean sendEnd = VI_OFF;

    *retCount = 0L;
    if ((sourceFile = fopen (filename, "rb")) == VI_NULL)
        return VI_ERROR_INSTR_FILE_OPEN; /* not defined by VTL */

    while (!feof (sourceFile)) {
        bytesRead = (ViUInt32)fread (buffer, sizeof (ViByte), BUFFER_SIZE, sourceFile);
        if ((writeBytes > BUFFER_SIZE) && (bytesRead == BUFFER_SIZE)) {
            viGetAttribute (instrSession, VI_ATTR_SEND_END_EN, &sendEnd);
            viSetAttribute (instrSession, VI_ATTR_SEND_END_EN, VI_OFF);
            rssme_status = viWrite (instrSession, buffer, BUFFER_SIZE, &bytesWritten);
            viSetAttribute (instrSession, VI_ATTR_SEND_END_EN, sendEnd);
            writeBytes -= BUFFER_SIZE;
            *retCount += bytesWritten;
            if (rssme_status < 0)
                break;
        }
        else {
            rssme_status = viWrite (instrSession, buffer, ((bytesRead < writeBytes) ? bytesRead : writeBytes), &bytesWritten);
            *retCount += bytesWritten;
            break;
        }
    }

    fclose (sourceFile);
    return rssme_status;
}

/*****************************************************************************/
/*----------- INSTRUMENT-DEPENDENT UTILITY ROUTINES -------------------------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the rssme_reset operation and by the      */
/*           rssme_init function if the reset option has not been            */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/
/*===========================================================================*/
ViStatus rssme_defaultInstrSetup (ViSession instrSession)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViInt16 i = 0;
    ViChar opt[100];
    rssme_instrRange instrPtr;
    
    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    if ((rssme_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssme_status;
    
    if (instrPtr == NULL) 
        instrPtr = malloc (sizeof (struct rssme_statusDataRanges));

    instrPtr -> option = 0;
    Fmt (instrPtr -> instrDriverRevision, "%s<%s", RSSME_REVISION);

    if (viSetAttribute (instrSession, VI_ATTR_USER_DATA, (ViUInt32)instrPtr) != 0)
        return rssme_status;

    /*----------- Clear Status Structures ---------------------------------*/
    if ((rssme_status = viWrite (instrSession, (ViBuf)"*CLS", 4, &retCnt)) < 0)
        return rssme_status;

    /*----------- Instrument Version Identification Query -----------------*/
    if ((rssme_status = viWrite (instrSession, (ViBuf)"*IDN?", 5, &retCnt)) < 0)
        return rssme_status;
    if ((rssme_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rssme_status;

    if (Scan (buffer, "%s>%s", opt) < 0) {
        rssme_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return rssme_status;
    }

    for (i=0; i<3; i++)
        if (FindPattern (opt, 0, -1, instrVersionList[i], 0, 0) >= 0)
            instrPtr -> maxFrequency = maxFrequencyList[i];

    /*----------- Option Identification Query -----------------------------*/
    if ((rssme_status = rssme_confOPT (instrSession, opt)) < 0)
        return rssme_status;

    for (i=0; i<rssme_NUMB_OF_AVAILABLE_OPTIONS; i++)
        if (FindPattern (opt, 0, -1, optionsList[i], 0, 0) >= 0)
            instrPtr -> option = (ViInt16)(instrPtr -> option + (ViInt16)pow(2,i));

    return rssme_status;
}

/*=========================================================================*/
/* Function: Status                                                        */
/* Purpose:  This function reads the Standard Event Register and checks    */
/*           for error.                                                    */
/*=========================================================================*/
ViStatus rssme_sysStatus (ViSession instrSession)
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;

    ViInt32 esr;

    /*--------------- Read the Standard Event Register --------------------*/
    if ((rssme_status = viWrite (instrSession, (ViBuf)"*ESR?", 5, &retCnt)) < 0)
        return rssme_status;
    if ((rssme_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rssme_status;

    if (Scan (buffer, "%s>%i[b4]", &esr) != 1) {
        rssme_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return rssme_status;
    }

    /*---------------- Check if any error bit was asserted ----------------*/
    if ((esr & 0x3C) != 0)
        rssme_status = rssme_ERROR_INSTRUMENT_ERROR;
    else
       rssme_status = VI_SUCCESS;

    return rssme_status;
}

/*=========================================================================*/
/* Function: Query Instrument Version                                      */
/* Purpose:  This function queries the instrument version.                 */
/*=========================================================================*/
ViStatus rssme_queryInstrVersion (ViSession instrSession, ViPReal64 maxFreq)
{
    ViStatus rssme_status = VI_SUCCESS;
    rssme_instrRange instrPtr;

    if ((rssme_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssme_status;

    *maxFreq = instrPtr -> maxFrequency;

    return rssme_status;
}

/*=========================================================================*/
/* Function: Query Instrument Option                                       */
/* Purpose:  This function checks if an option is included in the          */
/*           instrument.                                                   */
/*=========================================================================*/
ViStatus rssme_queryInstrOption (ViSession instrSession, ViInt16 queryBit, ViPInt16 bitValue)
{
    ViStatus rssme_status = VI_SUCCESS;
    rssme_instrRange instrPtr;

    if ((rssme_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rssme_status;

    if ((instrPtr -> option & queryBit) != 0)
        *bitValue = VI_ON;
    else
        *bitValue = VI_OFF;

    return rssme_status;
}

/*===========================================================================*/
/* Function: Instrument Options                                              */
/* Purpose:  This function returns the list of options which the instrument  */
/*           is equipped with.                                               */
/*===========================================================================*/
ViStatus rssme_confOPT (ViSession instrSession, ViChar optsList[])
{
    ViStatus rssme_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViBoolean i = VI_ON;
    ViInt32 searchIndex = 0;
    ViInt32 storeIndex = 0;

    FillBytes ((ViChar *)buffer, 0, 2048, '\0');

    if ((rssme_status = viWrite (instrSession, (ViBuf)"*OPT?", 5, &retCnt)) < 0)
        return rssme_status;
    if ((rssme_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rssme_status;

    if (Scan (buffer, "%s>%s", optsList) != 1) {
        rssme_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return rssme_status;
    }

    while (i) {

        searchIndex = FindPattern (optsList, searchIndex, -1, "S", 0, 0);

        if (searchIndex < 0) {
            FillBytes (optsList, storeIndex - 1, 1, 0);
            break;
        }

        CopyString (optsList, storeIndex, optsList, searchIndex, -1);

        storeIndex = FindPattern (optsList, storeIndex, -1, ",", 0, 0);

        if (storeIndex < 0) {
            storeIndex = StringLength (optsList);
            FillBytes (optsList, storeIndex - 1, 1, 0);
            break;
        }

        storeIndex++;

        searchIndex = storeIndex;
    }

    return rssme_status;
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
